/*
 * ColumnFile.h - Parse a file that contains column-based data.
 *
 * Copyright (c) 1999, Rhys Weatherley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rhys Weatherley.
 * 4. The name of Rhys Weatherley may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__ColumnFile_H__
#define	__ColumnFile_H__

class ColumnFileParser
{
public:

	ColumnFileParser();
	~ColumnFileParser();

private:

	FILE		*file;
	int			 linesLeftToSkip;
	char		 buffer[BUFSIZ];
	char        *columns[100];
	const char  *separators;

protected:

	// Callback from "process" for the next line of data.
	virtual void nextLine(char **columns);

public:

	// Set the list of separators to use (other than white space).
	void setSeparators(const char *sep) { separators = sep; }

	// Open a file for column-based parsing.
	int openFile(const char *filename, int skipLines);

	// Open a pipe to a command for column-based parsing.
	int openPipe(const char *command, int skipLines);

	// Close the current file.
	void close(void);

	// Read the next set of columns from the file.
	// Returns the number of columns, or -1 at EOF.
	int readColumns(char **& columns);

	// Process the entire file's contents, passing each line of
	// columns to "nextLine" in turn.  Any lines that have less
	// than the specified number of columns are padded with
	// blank columns.
	void process(int columns);

	// Convert a hexadecimal string value into a number.
	static unsigned long hexToNum(const char *str);

	// Convert a hexadecimal string value into an IP address.
	static char *hexToIP(const char *str);

};

#endif	/* __ColumnFile_H__ */
