/*
 * DialogBox.h - Base class for dialog box handling.
 *
 * Copyright (c) 1999, Rhys Weatherley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rhys Weatherley.
 * 4. The name of Rhys Weatherley may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__DialogBox_H__
#define	__DialogBox_H__

#define	DIALOG_BUTTON_NONE			0
#define	DIALOG_BUTTON_OK			1
#define	DIALOG_BUTTON_CANCEL		2
#define	DIALOG_BUTTON_CLOSE			4

typedef struct
{
	GtkWidget	*button;
	int			 buttonNum;

} DialogButton;

class DialogBox
{
public:

	DialogBox(GtkWidget *parent,
			  const char *title,
			  int stdbuttons=DIALOG_BUTTON_OK | DIALOG_BUTTON_CANCEL,
			  int modal=1);
	virtual ~DialogBox(void);

private:

	static void cb_dialog_ok(GtkWidget *widget, gpointer data);
	static void cb_dialog_cancel(GtkWidget *widget, gpointer data);
	static void cb_dialog_close(GtkWidget *widget, gpointer data);
	static void cb_dialog_wmclose(GtkWidget *widget, GdkEventAny *event,
								  gpointer data);
	static gint cb_dialog_key_pressed(GtkWidget *widget, GdkEventKey *event,
									  gpointer data);
	static void cb_dialog_button(GtkWidget *widget, gpointer data);
	static void cb_dialog_activate(GtkWidget *widget, gpointer data);

protected:

	// Widgets.
	GtkWidget		*dialog;	// Dialog container frame.
	GtkWidget		*vbox;		// Vertical box holding the dialog's contents.
	GtkWidget		*buttons;	// Horizontal box holding the buttons.
	GtkWidget		*okbutton;	// Widget for the OK button.
	GtkWidget		*cancelbutton; // Widget for the Cancel button.
	GtkWidget		*closebutton;  // Widget for the Close button.
	GtkWidget		*defaultbutton; // The current default button.
	gint			 insert;	// Position to insert extra buttons.

	// Callbacks for standard buttons.
	virtual void ok(void);
	virtual void cancel(void);
	virtual void close(void);
	virtual void wmclose(void);

	// Callbacks for user-defined buttons.
	virtual void button(GtkWidget *buttonWidget);

	// Activate the default button.
	void activateDefaultButton(void);

public:

	// Add another button to the dialog box.
	GtkWidget *addButton(const char *name, int isDefault=0);

	// Add a text entry field to this dialog box.
	GtkWidget *addText(int editable=1);

	// Show the dialog box.
	void show(int showcontents=1);

	// Hide the dialog box.
	void hide(void);

	// Close and destroy the dialog box, together with this object.
	void closeDialog(void);

	// Set a widget sensitive or insensitive.
	static void setSensitive(GtkWidget *widget, int flag)
		{ gtk_widget_set_sensitive(widget, flag); }

	// Determine if a widget is sensitive.
	static int isSensitive(GtkWidget *widget)
		{ return GTK_WIDGET_SENSITIVE(widget); }

	// Make a particular button the default button.
	void setDefault(GtkWidget *button)
		{ gtk_widget_grab_default(button); defaultbutton = button; }

	// Set the widget that will initially have the focus in this dialog box.
	void setFocus(GtkWidget *widget)
		{ gtk_window_set_focus(GTK_WINDOW(dialog), widget); }

	// Determine if the contents of a text entry widget are empty.
	static int textEmpty(GtkWidget *widget);

};

#endif	/* __DialogBox_H__ */
