/*
	EthernetTypes.cc - List of Ethernet packet payload types and names.

	This list was originally extracted from the one at the following URL:

		ftp://ftp.cavebear.com/pub/Ethernet.txt

	See that file for up to date copies and a description of the origins
	of this information.  This version of the list was extracted from
	the 1999/03/09 11:36:19 version of the file on that FTP site.
	The original file appears to be public domain in origin.  Apologies
	to the original author if it isn't public domain.

	Because this source file is based in large part on someone else's list,
	Rhys Weatherley makes no Copyright claim over this file.  It should be
	considered public domain and other people are free to use it as they
	see fit.  This in no way reduces Rhys Weatherley's Copyright claims on
	other source files within the application.
*/

typedef struct
{
	unsigned short	type;
	const char     *name;

} EthernetType;

static EthernetType types[] = {
	{0x0200, "Xerox PUP"},
	{0x0201, "Xerox PUP"},
	{0x0400, "Nixdorf"},
	{0x0600, "Xerox NS IDP"},
	{0x0601, "XNS Address Translation (3Mb only)"},
	{0x0800, "Internet Protocol (IP)"},
	{0x0801, "X.75 Internet"},
	{0x0802, "NBS Internet"},
	{0x0803, "ECMA Internet"},
	{0x0804, "CHAOSnet"},
	{0x0805, "X.25 Level 3"},
	{0x0806, "Address Resolution Protocol (ARP)"},
	{0x0807, "XNS Compatibility"},
	{0x081C, "Symbolics Private"},
	{0x0888, "Xyplex"},
	{0x0889, "Xyplex"},
	{0x088A, "Xyplex"},
	{0x0900, "Ungermann-Bass network debugger"},
	{0x0A00, "Xerox IEEE802.3 PUP"},
	{0x0A01, "Xerox IEEE802.3 PUP Address Translation"},
	{0x0BAD, "Banyan Systems"},
	{0x0BAF, "Banyon VINES Echo"},
	{0x1000, "Berkeley Trailer negotiation"},
	{0x1001, "Berkeley Trailer encapsulation for IP"},
	{0x1002, "Berkeley Trailer encapsulation for IP"},
	{0x1003, "Berkeley Trailer encapsulation for IP"},
	{0x1004, "Berkeley Trailer encapsulation for IP"},
	{0x1005, "Berkeley Trailer encapsulation for IP"},
	{0x1006, "Berkeley Trailer encapsulation for IP"},
	{0x1007, "Berkeley Trailer encapsulation for IP"},
	{0x1008, "Berkeley Trailer encapsulation for IP"},
	{0x1009, "Berkeley Trailer encapsulation for IP"},
	{0x100A, "Berkeley Trailer encapsulation for IP"},
	{0x100B, "Berkeley Trailer encapsulation for IP"},
	{0x100C, "Berkeley Trailer encapsulation for IP"},
	{0x100D, "Berkeley Trailer encapsulation for IP"},
	{0x100E, "Berkeley Trailer encapsulation for IP"},
	{0x100F, "Berkeley Trailer encapsulation for IP"},
	{0x1234, "DCA - Multicast"},
	{0x1600, "VALID system protocol"},
	{0x1989, "Artificial Horizons (\"Aviator\" dogfight simulator [on Sun])"},
	{0x1995, "Datapoint Corporation (RCL lan protocol)"},
	{0x3C00, "3Com NBP virtual circuit datagram (like XNS SPP) not registered"},
	{0x3C01, "3Com NBP System control datagram not registered"},
	{0x3C02, "3Com NBP Connect request (virtual cct) not registered"},
	{0x3C03, "3Com NBP Connect repsonse not registered"},
	{0x3C04, "3Com NBP Connect complete not registered"},
	{0x3C05, "3Com NBP Close request (virtual cct) not registered"},
	{0x3C06, "3Com NBP Close response not registered"},
	{0x3C07, "3Com NBP Datagram (like XNS IDP) not registered"},
	{0x3C08, "3Com NBP Datagram broadcast not registered"},
	{0x3C09, "3Com NBP Claim NetBIOS name not registered"},
	{0x3C0A, "3Com NBP Delete Netbios name not registered"},
	{0x3C0B, "3Com NBP Remote adaptor status request not registered"},
	{0x3C0C, "3Com NBP Remote adaptor response not registered"},
	{0x3C0D, "3Com NBP Reset not registered"},
	{0x4242, "PCS Basic Block Protocol"},
	{0x424C, "Information Modes Little Big LAN diagnostic"},
	{0x4321, "THD - Diddle"},
	{0x4C42, "Information Modes Little Big LAN"},
	{0x5208, "BBN Simnet Private"},
	{0x6000, "DEC unassigned, experimental"},
	{0x6001, "DEC Maintenance Operation Protocol (MOP) Dump/Load Assistance"},
	{0x6002, "DEC Maintenance Operation Protocol (MOP) Remote Console"},
	{0x6003, "DECNET Phase IV, DNA Routing"},
	{0x6004, "DEC Local Area Transport (LAT)"},
	{0x6005, "DEC diagnostic protocol (at interface initialization?)"},
	{0x6006, "DEC customer protocol"},
	{0x6007, "DEC Local Area VAX Cluster (LAVC), System Communication Architecture (SCA)"},
	{0x6008, "DEC AMBER"},
	{0x6009, "DEC MUMPS"},
	{0x6010, "3Com Corporation"},
	{0x6011, "3Com Corporation"},
	{0x6012, "3Com Corporation"},
	{0x6013, "3Com Corporation"},
	{0x6014, "3Com Corporation"},
	{0x7000, "Ungermann-Bass download"},
	{0x7001, "Ungermann-Bass NIUs"},
	{0x7002, "Ungermann-Bass diagnostic/loopback"},
	{0x7003, "Ungermann-Bass ??? (NMC to/from UB Bridge)"},
	{0x7005, "Ungermann-Bass Bridge Spanning Tree"},
	{0x7007, "OS/9 Microware"},
	{0x7009, "OS/9 Net?"},
	{0x7020, "LRT (England) (now Sintrom)"},
	{0x7021, "LRT (England) (now Sintrom)"},
	{0x7022, "LRT (England) (now Sintrom)"},
	{0x7023, "LRT (England) (now Sintrom)"},
	{0x7024, "LRT (England) (now Sintrom)"},
	{0x7025, "LRT (England) (now Sintrom)"},
	{0x7026, "LRT (England) (now Sintrom)"},
	{0x7027, "LRT (England) (now Sintrom)"},
	{0x7028, "LRT (England) (now Sintrom)"},
	{0x7029, "LRT (England) (now Sintrom)"},
	{0x7030, "Racal-Interlan"},
	{0x7031, "Prime NTS (Network Terminal Service)"},
	{0x7034, "Cabletron"},
	{0x8003, "Cronus VLN"},
	{0x8004, "Cronus Direct"},
	{0x8005, "HP Probe protocol"},
	{0x8006, "Nestar"},
	{0x8008, "AT&T/Stanford Univ.	Local use"},
	{0x8010, "Excelan"},
	{0x8013, "Silicon Graphics diagnostic"},
	{0x8014, "Silicon Graphics network games"},
	{0x8015, "Silicon Graphics reserved"},
	{0x8016, "Silicon Graphics XNS NameServer, bounce server"},
	{0x8019, "Apollo DOMAIN"},
	{0x802E, "Tymshare"},
	{0x802F, "Tigan, Inc."},
	{0x8035, "Reverse Address Resolution Protocol (RARP)"},
	{0x8036, "Aeonic Systems"},
	{0x8037, "IPX (Novell Netware?)"},
	{0x8038, "DEC LanBridge Management"},
	{0x8039, "DEC DSM/DDP"},
	{0x803A, "DEC Argonaut Console"},
	{0x803B, "DEC VAXELN"},
	{0x803C, "DEC DNS Naming Service"},
	{0x803D, "DEC Ethernet CSMA/CD Encryption Protocol"},
	{0x803E, "DEC Distributed Time Service"},
	{0x803F, "DEC LAN Traffic Monitor Protocol"},
	{0x8040, "DEC PATHWORKS DECnet NETBIOS Emulation"},
	{0x8041, "DEC Local Area System Transport"},
	{0x8042, "DEC unassigned"},
	{0x8044, "Planning Research Corp."},
	{0x8046, "AT&T"},
	{0x8047, "AT&T"},
	{0x8048, "DEC Availability Manager for Distributed Systems DECamds (but someone at DEC says not)"},
	{0x8049, "ExperData"},
	{0x805B, "VMTP (Versatile Message Transaction Protocol, RFC-1045) (Stanford) [was Stanford V Kernel, experimental]"},
	{0x805C, "Stanford V Kernel, version 6.0"},
	{0x805D, "Evans & Sutherland"},
	{0x8060, "Little Machines"},
	{0x8062, "Counterpoint Computers"},
	{0x8065, "University of Mass. at Amherst"},
	{0x8066, "University of Mass. at Amherst"},
	{0x8067, "Veeco Integrated Automation"},
	{0x8068, "General Dynamics"},
	{0x8069, "AT&T"},
	{0x806A, "Autophon"},
	{0x806C, "ComDesign"},
	{0x806D, "Compugraphic Corporation"},
	{0x806E, "8077	Landmark Graphics Corporation"},
	{0x807A, "Matra"},
	{0x807B, "Dansk Data Elektronik"},
	{0x807C, "Merit Internodal (or Univ of Michigan?)"},
	{0x807D, "Vitalink Communications"},
	{0x807E, "Vitalink Communications"},
	{0x807F, "Vitalink Communications"},
	{0x8080, "Vitalink TransLAN III Management"},
	{0x8081, "Counterpoint Computers"},
	{0x8082, "Counterpoint Computers"},
	{0x8083, "Counterpoint Computers"},
	{0x8088, "Xyplex"},
	{0x8089, "Xyplex"},
	{0x808A, "Xyplex"},
	{0x809B, "EtherTalk (AppleTalk over Ethernet)"},
	{0x809C, "Datability"},
	{0x809D, "Datability"},
	{0x809E, "Datability"},
	{0x809F, "Spider Systems Ltd."},
	{0x80A3, "Nixdorf Computers"},
	{0x80A4, "Siemens Gammasonics Inc."},
	{0x80A5, "Siemens Gammasonics Inc."},
	{0x80A6, "Siemens Gammasonics Inc."},
	{0x80A7, "Siemens Gammasonics Inc."},
	{0x80A8, "Siemens Gammasonics Inc."},
	{0x80A9, "Siemens Gammasonics Inc."},
	{0x80AA, "Siemens Gammasonics Inc."},
	{0x80AB, "Siemens Gammasonics Inc."},
	{0x80AC, "Siemens Gammasonics Inc."},
	{0x80AD, "Siemens Gammasonics Inc."},
	{0x80AE, "Siemens Gammasonics Inc."},
	{0x80AF, "Siemens Gammasonics Inc."},
	{0x80B0, "Siemens Gammasonics Inc."},
	{0x80B1, "Siemens Gammasonics Inc."},
	{0x80B2, "Siemens Gammasonics Inc."},
	{0x80B3, "Siemens Gammasonics Inc."},
	{0x80C0, "DCA (Digital Comm. Assoc.) Data Exchange Cluster"},
	{0x80C1, "DCA (Digital Comm. Assoc.) Data Exchange Cluster"},
	{0x80C2, "DCA (Digital Comm. Assoc.) Data Exchange Cluster"},
	{0x80C3, "DCA (Digital Comm. Assoc.) Data Exchange Cluster"},
	{0x80C6, "Pacer Software"},
	{0x80C7, "Applitek Corporation"},
	{0x80C8, "Intergraph Corporation"},
	{0x80C9, "Intergraph Corporation"},
	{0x80CA, "Intergraph Corporation"},
	{0x80CB, "Intergraph Corporation"},
	{0x80CC, "Intergraph Corporation"},
	{0x80CD, "Harris Corporation"},
	{0x80CE, "Harris Corporation"},
	{0x80CF, "Taylor Instrument"},
	{0x80D0, "Taylor Instrument"},
	{0x80D1, "Taylor Instrument"},
	{0x80D2, "Taylor Instrument"},
	{0x80D3, "Rosemount Corporation"},
	{0x80D4, "Rosemount Corporation"},
	{0x80D5, "IBM SNA Services over Ethernet"},
	{0x80DD, "Varian Associates"},
	{0x80DE, "TRFS (Integrated Solutions Transparent Remote File System)"},
	{0x80DF, "TRFS (Integrated Solutions Transparent Remote File System)"},
	{0x80E0, "Allen-Bradley"},
	{0x80E1, "Allen-Bradley"},
	{0x80E2, "Allen-Bradley"},
	{0x80E3, "Allen-Bradley"},
	{0x80E4, "Datability"},
	{0x80E5, "Datability"},
	{0x80E6, "Datability"},
	{0x80E7, "Datability"},
	{0x80E8, "Datability"},
	{0x80E9, "Datability"},
	{0x80EA, "Datability"},
	{0x80EB, "Datability"},
	{0x80EC, "Datability"},
	{0x80ED, "Datability"},
	{0x80EE, "Datability"},
	{0x80EF, "Datability"},
	{0x80F0, "Datability"},
	{0x80F2, "Retix"},
	{0x80F3, "AppleTalk Address Resolution Protocol (AARP)"},
	{0x80F4, "Kinetics"},
	{0x80F5, "Kinetics"},
	{0x80F7, "Apollo Computer"},
	{0x80FF, "Wellfleet Communications"},
	{0x8100, "Wellfleet Communications"},
	{0x8101, "Wellfleet Communications"},
	{0x8102, "Wellfleet; BOFL (Breath OF Life) pkts [every 5-10 secs.]"},
	{0x8103, "Wellfleet Communications"},
	{0x8107, "Symbolics Private"},
	{0x8108, "Symbolics Private"},
	{0x8109, "Symbolics Private"},
	{0x812B, "Talaris"},
	{0x8130, "Waterloo Microsystems Inc."},
	{0x8131, "VG Laboratory Systems"},
	{0x8137, "Novell (old) NetWare IPX (ECONFIG E option)"},
	{0x8138, "Novell, Inc."},
	{0x8139, "KTI"},
	{0x813A, "KTI"},
	{0x813B, "KTI"},
	{0x813C, "KTI"},
	{0x813D, "KTI"},
	{0x813F, "M/MUMPS data sharing"},
	{0x8145, "Vrije Universiteit (NL) Amoeba 4 RPC (obsolete)"},
	{0x8146, "Vrije Universiteit (NL) FLIP (Fast Local Internet Protocol)"},
	{0x8147, "Vrije Universiteit (NL) [reserved]"},
	{0x814C, "SNMP over Ethernet (see RFC1089)"},
	{0x814F, "Technically Elite Concepts Network Professor"},
	{0x817D, "XTP"},
	{0x8191, "PowerLAN NetBIOS/NetBEUI (PC)"},
	{0x81D6, "Artisoft Lantastic"},
	{0x81D7, "Artisoft Lantastic"},
	{0x8203, "QNX Software Systems Ltd."},
	{0x8204, "QNX Software Systems Ltd."},
	{0x8205, "QNX Software Systems Ltd."},
	{0x8390, "Accton Technologies (unregistered)"},
	{0x852B, "Talaris multicast"},
	{0x8582, "Kalpana"},
	{0x86DD, "Internet Protocol Version 6 (IPv6)"},
	{0x8739, "Control Technology Inc. RDP Without IP"},
	{0x873A, "Control Technology Inc. Mcast Industrial Ctrl Proto."},
	{0x873B, "Control Technology Inc. Proprietary"},
	{0x873C, "Control Technology Inc. Proprietary"},
	{0x8820, "Hitachi Cable (Optoelectronic Systems Laboratory)"},
	{0x8856, "Axis Communications AB proprietary bootstrap/config"},
	{0x8888, "HP LanProbe test?"},
	{0x9000, "Loopback (Configuration Test Protocol)"},
	{0x9001, "3Com (Formerly Bridge Communications), XNS Systems Management"},
	{0x9002, "3Com (Formerly Bridge Communications), TCP/IP Systems Management"},
	{0x9003, "3Com (Formerly Bridge Communications), loopback detection"},
	{0xAAAA, "DECNET? Used by VAX 6220 DEBNI"},
	{0xFAF5, "Sonix Arpeggio"},
	{0xFF00, "BBN VITAL-LanBridge cache wakeups"},
};
#define	num_types	(sizeof(types) / sizeof(EthernetType))

const char *EthernetTypeName(unsigned short type)
{
	int first = 0;
	int last = num_types - 1;
	int middle;
	unsigned short temptype;
	while(first <= last)
	{
		middle = (first + last) / 2;
		temptype = types[middle].type;
		if(temptype < type)
		{
			first = middle + 1;
		}
		else if(temptype > type)
		{
			last = middle - 1;
		}
		else
		{
			return types[middle].name;
		}
	}
	return 0;
}
