/*
 * FilterCondition.h - Filter condition handling.
 *
 * Copyright (c) 1999, Rhys Weatherley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rhys Weatherley.
 * 4. The name of Rhys Weatherley may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__FilterCondition_H__
#define	__FilterCondition_H__

typedef enum
{
	FilterAddressType_None,
	FilterAddressType_To,
	FilterAddressType_From,
	FilterAddressType_ToOrFrom,
	FilterAddressType_Between,

} FilterAddressType;

typedef enum
{
	FilterProtocolType_None          = 0x00,
	FilterProtocolType_UDP           = 0x01,
	FilterProtocolType_TCP           = 0x02,
	FilterProtocolType_ICMP          = 0x04,
	FilterProtocolType_OtherIP       = 0x08,
	FilterProtocolType_ARP           = 0x10,
	FilterProtocolType_RARP          = 0x20,
	FilterProtocolType_OtherEthernet = 0x40,
	FilterProtocolType_All           = 0x7F,

} FilterProtocolType;

class FilterCondition
{
public:

	FilterCondition();
	~FilterCondition();

public:

	FilterAddressType addrType;
	char             *addrTo;
	char             *addrFrom;
	char             *addrToOrFrom;
	char             *addrBetween;
	char             *addrAnd;

	FilterProtocolType protocols;

	int				 portEnabled;
	char            *ports;

public:

	// Set various string fields, destroying the old values.
	void setAddrTo(const char *value);
	void setAddrFrom(const char *value);
	void setAddrToOrFrom(const char *value);
	void setAddrBetween(const char *value);
	void setAddrAnd(const char *value);
	void setPorts(const char *value);

	// Check the state of a protocol bit.
	int hasProtocol(FilterProtocolType type)
		{ return ((protocols & type) != 0); }

	// Set a protocol bit.
	void setProtocol(FilterProtocolType type)
		{ protocols = (FilterProtocolType)(((int)protocols) | ((int)type)); }

	// Convert this filtering condition into a program that
	// can be passed to "pcap_compile".  Returns NULL if
	// no filtering should be applied.
	char *convert(int linkType);

};

#endif	/* __FilterCondition_H__ */
