/*
 * LinkFormatRule.h - Formatting rules for link types.
 *
 * Copyright (c) 1999, Rhys Weatherley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rhys Weatherley.
 * 4. The name of Rhys Weatherley may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__LinkFormatRule_H__
#define	__LinkFormatRule_H__

#include "FormatRule.h"

// Base class for link format rules.
class LinkFormatRule : public FormatRule
{
public:

	LinkFormatRule(FormatRuleTable *_table, unsigned long _tag);
	LinkFormatRule(FormatRuleTable *_table);
	virtual ~LinkFormatRule();

public:

	virtual void formatPacketShort(NetPacket& packet, FormatOutput& output);
	virtual void formatPacketLong(NetPacket& packet, FormatOutput& output);

};

// Format packets that arrived over an Ethernet link.
class EthernetLinkFormatRule : public LinkFormatRule
{
public:

	EthernetLinkFormatRule(FormatRuleTable *_table, unsigned long _tag);
	virtual ~EthernetLinkFormatRule();

public:

	virtual void formatPacketShort(NetPacket& packet, FormatOutput& output);
	virtual void formatPacketLong(NetPacket& packet, FormatOutput& output);

};

// Format packets that arrived over a raw link.
class RawLinkFormatRule : public LinkFormatRule
{
public:

	RawLinkFormatRule(FormatRuleTable *_table, unsigned long _tag);
	virtual ~RawLinkFormatRule();

public:

	virtual void formatPacketShort(NetPacket& packet, FormatOutput& output);
	virtual void formatPacketLong(NetPacket& packet, FormatOutput& output);

};

// Format packets that arrived over a PPP link.
class PPPLinkFormatRule : public LinkFormatRule
{
public:

	PPPLinkFormatRule(FormatRuleTable *_table, unsigned long _tag);
	virtual ~PPPLinkFormatRule();

public:

	virtual void formatPacketShort(NetPacket& packet, FormatOutput& output);
	virtual void formatPacketLong(NetPacket& packet, FormatOutput& output);

};

// Format packets that arrived over a null link.
class NullLinkFormatRule : public LinkFormatRule
{
public:

	NullLinkFormatRule(FormatRuleTable *_table, unsigned long _tag);
	virtual ~NullLinkFormatRule();

public:

	virtual void formatPacketShort(NetPacket& packet, FormatOutput& output);
	virtual void formatPacketLong(NetPacket& packet, FormatOutput& output);

};

// Format packets that arrived over an unknown link.
class DefaultLinkFormatRule : public LinkFormatRule
{
public:

	DefaultLinkFormatRule(FormatRuleTable *_table);
	virtual ~DefaultLinkFormatRule();

public:

	virtual void formatPacketShort(NetPacket& packet, FormatOutput& output);
	virtual void formatPacketLong(NetPacket& packet, FormatOutput& output);

};

// Pseudo data link types for well-known payload types.
#define	PSEUDO_DLT_IP			0x1000
#define	PSEUDO_DLT_ARP			0x1001
#define	PSEUDO_DLT_RARP			0x1002

#endif	/* __LinkFormatRule_H__ */
