/*
 * MenuUtil.c - Menu utilities that I need that GNOME doesn't have yet.
 *
 * Copyright (c) 1999, Rhys Weatherley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rhys Weatherley.
 * 4. The name of Rhys Weatherley may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <gnome.h>
#include <string.h>
#include "MenuUtil.h"

#ifdef	__cplusplus
extern	"C" {
#endif

void menu_util_process_items(GnomeApp *app, MenuStateInfo *info,
							 ToolbarMappingInfo *toolbarInfo,
							 void *data)
{
	GtkWidget *menubar;
	GtkToolbar *toolbar;
	GtkWidget *shell;
	GtkWidget *item;
	GtkWidget *button;
	GnomeDockLayoutItem *layoutItem;
	GnomeDockItem *dockItem;
	GnomeDockPlacement placementReturn;
	guint numBandReturn;
	guint bandPositionReturn;
	guint offsetReturn;
	gint position;
	gboolean enabled;
	gboolean checked;

	/* Sanity check the arguments */
	if(!app || !info)
	{
		return;
	}
	menubar = app->menubar;
	if(!menubar)
	{
		return;
	}

	/* Find the toolbar that is attached to the application */
	if(app->layout)
	{
		layoutItem = gnome_dock_layout_get_item_by_name(app->layout,
														GNOME_APP_TOOLBAR_NAME);
		if(layoutItem)
			toolbar = GTK_TOOLBAR(gnome_dock_item_get_child(layoutItem->item));
		else
			toolbar = NULL;
	}
	else if(app->dock)
	{
		dockItem = gnome_dock_get_item_by_name(GNOME_DOCK(app->dock),
											   GNOME_APP_TOOLBAR_NAME,
											   &placementReturn,
											   &numBandReturn,
											   &bandPositionReturn,
											   &offsetReturn);
		if(dockItem)
			toolbar = GTK_TOOLBAR(gnome_dock_item_get_child(dockItem));
		else
			toolbar = NULL;
	}
	else
	{
		toolbar = NULL;
	}

	/* Scan the menu state directives */
	while(info->menuPath != NULL)
	{
		/* Find the menu item widget */
		shell = gnome_app_find_menu_pos(menubar, _(info->menuPath), &position);
		if(shell)
		{
			item = (GtkWidget *)(g_list_nth_data
							(GTK_MENU_SHELL(shell)->children, position - 1));
		}
		else
		{
			item = NULL;
		}

		/* Find the toolbar button widget */
		if(toolbar && toolbarInfo)
		{
			position = 0;
			while(toolbarInfo[position].menuPath != NULL &&
			      strcmp(toolbarInfo[position].menuPath, info->menuPath))
			{
				++position;
			}
			if(toolbarInfo[position].menuPath != NULL &&
			   toolbarInfo[position].toolbarButton >= 0 &&
			   toolbarInfo[position].toolbarButton < toolbar->num_children)
			{
				button = ((GtkToolbarChild *)(g_list_nth_data
						(toolbar->children,
						 toolbarInfo[position].toolbarButton)))->widget;
			}
			else
			{
				button = NULL;
			}
		}
		else
		{
			button = NULL;
		}

		/* Determine the new states for the item */
		if(info->enableCallback)
			enabled = (*(info->enableCallback))(data);
		else
			enabled = TRUE;
		if(info->checkedCallback)
			checked = (*(info->checkedCallback))(data);
		else
			checked = FALSE;

		/* Set the new menu and toolbar button states */
		if(item)
		{
			gtk_widget_set_sensitive(item, enabled);
			gtk_menu_item_configure(GTK_MENU_ITEM(item), checked,
									(GTK_MENU_ITEM(item)->submenu != NULL));
		}
		if(button)
		{
			gtk_widget_set_sensitive(button, enabled);
		}
		++info;
	}
}

gboolean menu_util_is_enabled(MenuStateInfo *info,
							  const char *path, void *data)
{
	while(info && info->menuPath != NULL)
	{
		if(!strcmp(info->menuPath, path))
		{
			if(info->enableCallback)
				return (*(info->enableCallback))(data);
			else
				return TRUE;
		}
		++info;
	}
	return TRUE;
}

gboolean menu_util_is_checked(MenuStateInfo *info,
							  const char *path, void *data)
{
	while(info && info->menuPath != NULL)
	{
		if(!strcmp(info->menuPath, path))
		{
			if(info->checkedCallback)
				return (*(info->checkedCallback))(data);
			else
				return FALSE;
		}
		++info;
	}
	return FALSE;
}

#ifdef	__cplusplus
};
#endif
