/*
 * MenuUtil.h - Menu utilities that I need that GNOME doesn't have yet.
 *
 * Copyright (c) 1999, Rhys Weatherley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rhys Weatherley.
 * 4. The name of Rhys Weatherley may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__MENU_UTIL_H__
#define	__MENU_UTIL_H__

#ifdef	__cplusplus
extern	"C" {
#endif

/*
 * Function prototype for "enabled" and "checked" callbacks.
 */
typedef gboolean (*MenuStateCallback)(void *data);

/*
 * Declare an "menu state information" entry.  This declares
 * information that can be used to determine the "greyed out"
 * and "checked" state of all menu items.
 *
 *		"menuPath" is the path to the menu item.
 *		"enableCallback" is the function to call to determine
 *				   if the menu item or toolbar button is enabled.
 *		"checkedCallback" is the function to call to determine
 *				   if the menu item is checked.
 */
typedef struct
{
	gchar			   *menuPath;
	MenuStateCallback	enableCallback;
	MenuStateCallback	checkedCallback;

} MenuStateInfo;

/*
 * Declare a toolbar to menu path mapping.  This declares
 * information that can be used to associate "greying out"
 * or "checking" a menu item with a corresponding toolbar
 * button.
 *
 *		"menuPath" is the path to the menu item.
 *		"toolbarButton" is the position of the toolbar button.
 */
typedef struct
{
	gchar			   *menuPath;
	gint				toolbarButton;

} ToolbarMappingInfo;

/*
 * Helper macros.
 */
#define	MENUSTATEINFO(path,enablecb,checkedcb)	\
			{(path), (MenuStateCallback)(enablecb), \
			 (MenuStateCallback)(checkedcb)}
#define	MENUSTATEINFO_END	\
			{NULL, NULL, NULL}
#define	TOOLBARMAPPINGINFO(path,button)	\
			{(path), (button)}
#define	TOOLBARMAPPINGINFO_END	\
			{NULL, -1}

/*
 * Scan a list of item state records and activate any changes
 * that are indicated by it.  Menu items that are not mentioned
 * by the "info" list are left as-is.
 */
void menu_util_process_items(GnomeApp *app, MenuStateInfo *info,
							 ToolbarMappingInfo *toolbarInfo,
							 void *data);

/*
 * Determine if a particular menu item is enabled, by calling
 * its corresponding "enableCallback" function.  This will return
 * TRUE if the item is not in the list, or if it doesn't have
 * a callback.
 */
gboolean menu_util_is_enabled(MenuStateInfo *info,
							  const char *path, void *data);

/*
 * Determine if a particular menu item is checked, by calling
 * its corresponding "checkedCallback" function.  This will return
 * FALSE if the item is not in the list, or if it doesn't have
 * a callback.
 */
gboolean menu_util_is_checked(MenuStateInfo *info,
							  const char *path, void *data);

#ifdef	__cplusplus
};
#endif

#endif	/* __ENU_UTIL_H__ */
