/*
 * PingDialog.h - Dialog box code for the ping functionality.
 *
 * Copyright (c) 1999, Rhys Weatherley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rhys Weatherley.
 * 4. The name of Rhys Weatherley may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__PingDialog_H__
#define	__PingDialog_H__

#include "PingSupport.h"
#include "DialogBox.h"

class PingDialogBox;

class PingDialogSupport : public PingSupport
{
public:

	PingDialogSupport(PingDialogBox *_dlog);
	virtual ~PingDialogSupport();

private:

	PingDialogBox *dlog;
	guint		   timeoutId;
	gint		   selectId;

	static gboolean cb_timeout_proc(gpointer data);
	void timeoutFired(void) { timeoutCallback(); }
	static void cb_select_proc(gpointer data, gint source,
							   GdkInputCondition cond);	
	void selectFired(void) { selectCallback(); }

protected:

	// Inherited methods.
	virtual void registerTimeout(int nsecs);
	virtual void unregisterTimeout(void);
	virtual void registerSelect(void);
	virtual void unregisterSelect(void);
	virtual void pingResponse(struct in_addr& addr, unsigned seqnum,
							  unsigned ttl, struct timeval& roundTrip);
	virtual int  reportError(void);
	virtual void reportStats(unsigned long sent, unsigned long received,
							 struct timeval& minTrip, struct timeval& maxTrip,
							 struct timeval& totalTrip);
	virtual void stopping(void);

};

class PingDialogBox : public DialogBox
{
	friend class PingDialogSupport;

public:

	PingDialogBox(GtkWidget *parent);
	virtual ~PingDialogBox();

private:

	PingDialogSupport	pinger;
	guint				startTimeoutId;
	struct in_addr		pingAddr;
	GtkWidget		   *hostnameLabel;
	GtkWidget		   *hostname;
	GtkWidget		   *replies;
	GtkWidget		   *status;
	GtkWidget		   *startbutton;
	GtkWidget		   *stopbutton;

	// Record a change in the hostname field.
	static void cb_hostname_changed(GtkWidget *widget, PingDialogBox *box);
	void hostnameChanged(void);

	// Timeout handling to start the ping.
	static gboolean cb_start_timeout_proc(gpointer data);

	// Stop all ping activity.
	void stopPing(void);

protected:

	// Inherited methods.
	virtual void close(void);
	virtual void wmclose(void);
	virtual void button(GtkWidget *buttonWidget);

};

#endif	/* __PingDialog_H__ */
