/*
 * PingSupport.h - Support code for doing network pings.
 *
 * Copyright (c) 1999, Rhys Weatherley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rhys Weatherley.
 * 4. The name of Rhys Weatherley may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__PingSupport_H__
#define	__PingSupport_H__

#include "NetIncludes.h"

class PingSupport
{
public:

	PingSupport();
	virtual ~PingSupport();

private:

	int					haveTimeout;
	unsigned long		maxSent;
	unsigned long		numSent;
	unsigned long		numReceived;
	unsigned long		finalCount;
	struct sockaddr_in	address;
	unsigned short		identifier;
	static unsigned short global_identifier;
	struct timeval		minRoundTrip;
	struct timeval		maxRoundTrip;
	struct timeval		totalRoundTrip;

private:

	// Send a single ping packet.
	void sendPing(void);

protected:

	int					fd;

	// Register a timeout callback to call every "nsecs" seconds.
	virtual void registerTimeout(int nsecs) = 0;

	// Unregister the timeout callback.
	virtual void unregisterTimeout(void) = 0;

	// Function that should be called when a timeout occurs.
	void timeoutCallback(void);

	// Register the file descriptor for read selections.
	virtual void registerSelect(void) = 0;

	// Unregister the file descriptor.
	virtual void unregisterSelect(void) = 0;

	// Function that should be called when a read select occurs.
	void selectCallback(void);

	// Function that is called when a ping response is received.
	virtual void pingResponse(struct in_addr& addr, unsigned seqnum,
							  unsigned ttl, struct timeval& roundTrip);

	// Report an error (in errno) that is due to a failed send.
	// Return zero to abort the ping.
	virtual int reportError(void);

	// Report a change in the sent/received statistics.
	virtual void reportStats(unsigned long sent, unsigned long received,
							 struct timeval& minTrip, struct timeval& maxTrip,
							 struct timeval& totalTrip);

	// Report that the pinger is stopping after the maximum number of sends.
	virtual void stopping(void);

public:

	// Determine if we can ping (i.e. are we running as root or not?).
	static int canPing(void);

	// Send the initial ping to a given host.  Zero on error.
	// If "nsecs" is zero or negative, then only one ping will be sent.
	int ping(struct in_addr& addr, int nsecs, int maxToSend=100);

	// Stop pinging.
	void stop(void);

	// Determine if this object is in the process of pinging.
	int isPinging(void) const { return (fd != -1); }

};

#endif	/* __PingSupport_H__ */
