/*
 * PrintSupport.h - Support routines for printing packets.
 *
 * Copyright (c) 1999, Rhys Weatherley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rhys Weatherley.
 * 4. The name of Rhys Weatherley may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__PrintSupport_H__
#define	__PrintSupport_H__

#include "FormatRule.h"

class PrintFormatOutput : public FormatOutput
{
public:

	PrintFormatOutput(FormatRuleTable *_rules, int _maxPackets,
					  int _allowUnderlines=1);
	virtual ~PrintFormatOutput();

private:

	FormatRuleTable	   *rules;
	char			    numFormat[16];
	int					allowUnderlines;
	FILE			   *outstream;
	char	 			buffer[4096];
	int		 			buflen;
	int					firstSection;
	int32_t 			thiszone;
	int32_t 			today;
	int32_t 			prevday;
	char				datebuf[32];

public:

	// Inherited methods.
	virtual void outputShort(const char *format, ...);
	virtual void outputShortString(const char *string, int len);
	virtual void outputSection(const char *name);
	virtual void outputLong(const char *field, const char *format, ...);
	virtual void outputHex(const char *field, const void *data,
						   unsigned len);
	virtual void outputText(const char *field, const void *data,
						    unsigned len, int suppressCRs);
	virtual int wantLinkHeader(void);
	virtual void shortFormColor(FormatColor color);
	virtual int resolveIPAddresses(void);
	virtual int fullyQualifiedNames(void);
	virtual void markIPAddress(const void *addr);

	// Open a stream to a print command.
	int openCommand(const char *command);

	// Open a stream to a print file.
	int openFile(const char *filename);

	// Close the current print stream.
	void close(void);

	// Write a packet in short form to the print stream.
	int writeShort(NetPacket& packet);

	// Write a packet in long form to the print stream.
	int writeLong(NetPacket& packet);

};

#endif	/* __PrintSupport_H__ */
