/*
 * Resolver.h - Interface to the IP address to name resolver.
 *
 * Copyright (c) 1999, Rhys Weatherley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rhys Weatherley.
 * 4. The name of Rhys Weatherley may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__Resolver_H__
#define	__Resolver_H__

#include "NetIncludes.h"

// Start the resolver running.  It runs in the background, processing
// IP lookup requests from the front-end, and passing names back.
// Returns -1 if the resolver cannot be started, or a file descriptor
// to be selected on to process responses from the resolver.
int ResolverStart(char *argv0);

// Stop the resolver running.
void ResolverStop(void);

// Set the IP and netmask of the current network interface.  This is
// used to detect broadcast and subnet addresses, which are not resolved.
// If "resolveForeign" is zero, then addresses on other subnets will
// always be returned in numeric form.
void ResolverNetwork(struct in_addr address, struct in_addr netmask,
                     int resolveForeign);

// Change the status of the "resolveForeign" flag.
void ResolverChangeForeign(int resolveForeign);

// Look up an IP address to get the hostname associated with it.
// Returns the hostname if it is already known, or NULL if a
// request is pending, or if the address should be displayed
// as a numeric value.  If "suppressDomain" is non-zero, then
// don't return the domain name component of the name.
const char *ResolverLookup(struct in_addr address, int suppressDomain);

// Look up an IP address to get the hostname, but only if it is cached.
const char *ResolverLookupCached(struct in_addr address,
								 int suppressDomain);

// Add an (IP address, hostname) pair to the lookup cache.  This can
// be used to pre-prime the cache from DNS replies in a previously
// captured session.  This avoids the need to use a live DNS server
// to find hostnames when captured data is analysed offline.
void ResolverAddToCache(struct in_addr address, const char *hostname);

// Clear the resolver's hostname cache.
void ResolverClear(void);

// Handle a read select on the resolver's file descriptor.
void ResolverSelect(void);

// Function that is called when an asynchronous request finally returns,
// and the answer has been entered into the cache.  This is supplied by
// the user code.
void ResolverCallback(struct in_addr address);

// Function that is called when the resolver process disappears
// and the surrounding program should disable the select callback.
void ResolverDisappeared(int fd);

#endif	/* __Resolver_H__ */
