/*
 * TCPFormatRule.h - Formatting rules for TCP packets.
 *
 * Copyright (c) 1999, Rhys Weatherley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rhys Weatherley.
 * 4. The name of Rhys Weatherley may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__TCPFormatRule_H__
#define	__TCPFormatRule_H__

#include "FormatRule.h"

// Session information record.
typedef struct _TCPSessionInfo
{
	struct _TCPSessionInfo *next;
	unsigned long  addr;
	unsigned short port;
	unsigned short lookupPort;

} TCPSessionInfo;

// Format TCP packets.
class TCPFormatRule : public FormatRule
{
	friend class TCPPortFormatRule;

public:

	TCPFormatRule(FormatRuleTable *_table, unsigned long _tag);
	virtual ~TCPFormatRule();

private:

	FormatRuleTable portRules;
	TCPSessionInfo *sessions;

public:

	virtual void formatPacketShort(NetPacket& packet, FormatOutput& output);
	virtual void formatPacketLong(NetPacket& packet, FormatOutput& output);
	virtual void clearState(void);

};

// Base class for TCP port formatters.
class TCPPortFormatRule : public FormatRule
{
public:

	TCPPortFormatRule(TCPFormatRule *_tcp, FormatRuleTable *_table,
					  unsigned long _tag);
	virtual ~TCPPortFormatRule();

private:

	TCPFormatRule *tcp;

public:

	// Register a session with the TCP formatter.  "addr" and "port"
	// define the destination port to be recognized as a session.
	// "lookupPort" is the port number to use when looking up a
	// formatter to handle the TCP session packet.
	void registerSession(unsigned long addr, unsigned short port,
						 unsigned short lookupPort);

	// Unregister a session from the TCP formatter.
	void unregisterSession(unsigned long addr, unsigned short port,
						   unsigned short lookupPort);

};

#endif	/* __TCPFormatRule_H__ */
