/*
 * TelnetFormatRule.h - Formatting rules for telnet packets.  This formatter
 *                      can also be used to format simple text-based protocols
 *                      like SMTP, NNTP, POP3, etc.
 *
 * Copyright (c) 1999, Rhys Weatherley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rhys Weatherley.
 * 4. The name of Rhys Weatherley may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__TelnetFormatRule_H__
#define	__TelnetFormatRule_H__

#include "FormatRule.h"

// Format Telnet packets.
class TelnetFormatRule : public FormatRule
{
public:

	TelnetFormatRule(FormatRuleTable *_table, unsigned long _tag,
					 const char *_sectionName, int _telnetCommands,
					 int _suppressCRs);
	virtual ~TelnetFormatRule();

private:

	const char *sectionName;	// Long-form section name.
	int		    telnetCommands;	// TRUE to look for embedded telnet commands.
	int			suppressCRs;	// TRUE to suppress CR's from being printed.

	// Output a block of data as either text or binary.
	void outputBlock(FormatOutput& output, const void *data, unsigned len);

	// Format telnet option data into a buffer.  Returns the number
	// of bytes actually processed.
	unsigned formatOption(char *buf, int option,
						  const unsigned char *data, unsigned len);

public:

	virtual void formatPacketShort(NetPacket& packet, FormatOutput& output);
	virtual void formatPacketLong(NetPacket& packet, FormatOutput& output);

};

#endif	/* __TelnetFormatRule_H__ */
