/*
 * netpeek.cc - Main program entry point for the "netpeek" program.
 *
 * Copyright (c) 1999, Rhys Weatherley.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Rhys Weatherley.
 * 4. The name of Rhys Weatherley may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <gnome.h>
#include <gtk-xmhtml/gtk-xmhtml.h>
};	// Needed to end unmatched 'extern "C"' in <gtk-xmhtml/gtk-xmhtml.h>.
#include "NetMonitor.h"
#include "FormatRule.h"
#include "MenuUtil.h"
#include "FilterCondition.h"
#include "PingDialog.h"
#include "LookupDialog.h"
#include "ColumnFile.h"
#include "Resolver.h"
#include "PrintSupport.h"
#include <time.h>
#include <pcap-int.h>
#include <netinet/tcp.h>

#include "capture_start.xpm"
#include "capture_start_disabled.xpm"
#include "capture_stop.xpm"
#include "capture_stop_disabled.xpm"
#include "capture_start_small.xpm"
#include "capture_start_small_disabled.xpm"
#include "capture_stop_small.xpm"
#include "capture_stop_small_disabled.xpm"

static GnomeStockPixmapEntryData capture_start_regular =
	{GNOME_STOCK_PIXMAP_TYPE_DATA,
	 27, 20, NULL,
	 capture_start_xpm};
static GnomeStockPixmapEntryData capture_start_disabled =
	{GNOME_STOCK_PIXMAP_TYPE_DATA,
	 27, 20, NULL,
	 capture_start_disabled_xpm};
static GnomeStockPixmapEntryData capture_start_focused =
	{GNOME_STOCK_PIXMAP_TYPE_DATA,
	 27, 20, NULL,
	 capture_start_xpm};
static GnomeStockPixmapEntryData capture_stop_regular =
	{GNOME_STOCK_PIXMAP_TYPE_DATA,
	 27, 20, NULL,
	 capture_stop_xpm};
static GnomeStockPixmapEntryData capture_stop_disabled =
	{GNOME_STOCK_PIXMAP_TYPE_DATA,
	 27, 20, NULL,
	 capture_stop_disabled_xpm};
static GnomeStockPixmapEntryData capture_stop_focused =
	{GNOME_STOCK_PIXMAP_TYPE_DATA,
	 27, 20, NULL,
	 capture_stop_xpm};
static GnomeStockPixmapEntryData capture_start_small_regular =
	{GNOME_STOCK_PIXMAP_TYPE_DATA,
	 22, 16, NULL,
	 capture_start_small_xpm};
static GnomeStockPixmapEntryData capture_start_small_disabled =
	{GNOME_STOCK_PIXMAP_TYPE_DATA,
	 22, 16, NULL,
	 capture_start_small_disabled_xpm};
static GnomeStockPixmapEntryData capture_start_small_focused =
	{GNOME_STOCK_PIXMAP_TYPE_DATA,
	 22, 16, NULL,
	 capture_start_small_xpm};
static GnomeStockPixmapEntryData capture_stop_small_regular =
	{GNOME_STOCK_PIXMAP_TYPE_DATA,
	 22, 16, NULL,
	 capture_stop_small_xpm};
static GnomeStockPixmapEntryData capture_stop_small_disabled =
	{GNOME_STOCK_PIXMAP_TYPE_DATA,
	 22, 16, NULL,
	 capture_stop_small_disabled_xpm};
static GnomeStockPixmapEntryData capture_stop_small_focused =
	{GNOME_STOCK_PIXMAP_TYPE_DATA,
	 22, 16, NULL,
	 capture_stop_small_xpm};

int32_t gmt2local(time_t t);

#define	NETPEEK_NAME_CONFIG			"netpeek"
#define	NETPEEK_NAME_DISPLAY		_("NetPeek")
#define	NETPEEK_NAME_VERSION		"0.0.4"

class NetPeekSink;

GnomeClient *app_client;
GnomeApp *app_main_window;
GnomeAppBar *app_status_bar;
GtkWidget *app_contents;
GtkWidget *app_short_form;
GtkWidget *app_long_form;
GtkWidget *app_open_dialog;
GtkWidget *app_save_dialog;
gint app_input_select_fd = -1;
gint app_short_form_selected = -1;

NetMonitor global_net_monitor;
NetPeekSink *global_net_sink;
unsigned long global_net_packet_count = 0;
FilterCondition global_filter;

int pref_resolve_addresses = 1;
int pref_resolve_foreign = 1;
int pref_fully_qualified = 1;
int pref_hex_packets = 0;
int pref_capture_length = 2048;
int pref_auto_scroll = 1;

char *netstat_program = NULL;
char *arp_program = NULL;
char *ifconfig_program = NULL;
char *rpcinfo_program = NULL;

static void updateMenus(void);

#define	NETPEEK_MAX_ADDRESSES		2

class NetPeekFormat : public FormatOutput
{
	friend class NetPeekSink;

public:

	NetPeekFormat();
	virtual ~NetPeekFormat();

private:

	char buffer[4096];
	int buflen;
	FormatColor color;

	char *htmlbuf;
	int htmlbuflen;
	int htmlbufmax;
	int firstsection;

	GdkColor colors[10];
	int haveColors;

	bpf_u_int32 addresses[NETPEEK_MAX_ADDRESSES];
	int numAddresses;

private:

	// Clear the HTML buffer,
	void clearHTML(void);

	// Make extra space in the HTML buffer.
	int makeSpace(int needed);

	// Write data to the HTML buffer.
	void writeHTML(const char *data);

	// Write a string to the HTML buffer, after encoding special characters.
	void writeString(const char *str, int len=-1);

public:

	virtual void outputShort(const char *format, ...);
	virtual void outputShortString(const char *string, int len);
	virtual void outputSection(const char *name);
	virtual void outputLong(const char *field, const char *format, ...);
	virtual void outputHex(const char *field, const void *data,
						   unsigned len);
	virtual void outputText(const char *field, const void *data,
						    unsigned len, int suppressCRs);
	virtual int wantLinkHeader(void);
	virtual int resolveIPAddresses(void);
	virtual int fullyQualifiedNames(void);
	virtual void markIPAddress(const void *addr);
	virtual void shortFormColor(FormatColor color);

};

NetPeekFormat::NetPeekFormat()
{
	htmlbuf = 0;
	htmlbuflen = 0;
	htmlbufmax = 0;
	firstsection = 1;
	haveColors = 0;
	numAddresses = 0;
}

NetPeekFormat::~NetPeekFormat()
{
	if(htmlbuf)
	{
		free(htmlbuf);
	}
}

void NetPeekFormat::clearHTML(void)
{
	if(htmlbuf)
	{
		free(htmlbuf);
		htmlbuf = 0;
	}
	htmlbuflen = 0;
	htmlbufmax = 0;
	firstsection = 1;
}

int NetPeekFormat::makeSpace(int needed)
{
	int size = (htmlbuflen + needed + 1);
	if(size > htmlbufmax)
	{
		char *newbuf;
		size = (size + 4095) & ~4095;
		if((newbuf = (char *)realloc(htmlbuf, size)) == 0)
		{
			return 0;
		}
		htmlbuf = newbuf;
		htmlbufmax = size;
	}
	return 1;
}

void NetPeekFormat::writeHTML(const char *data)
{
	int len = strlen(data);
	if(makeSpace(len))
	{
		strcpy(htmlbuf + htmlbuflen, data);
		htmlbuflen += len;
	}
}

void NetPeekFormat::writeString(const char *str, int len)
{
	if(len < 0)
	{
		len = strlen(str);
	}
	if(!makeSpace(len))
	{
		return;
	}
	while(len > 0)
	{
		if(*str == '<')
		{
			if(!makeSpace(len + 3))
			{
				break;
			}
			strcpy(htmlbuf + htmlbuflen, "&lt;");
			htmlbuflen += 4;
		}
		else if(*str == '>')
		{
			if(!makeSpace(len + 3))
			{
				break;
			}
			strcpy(htmlbuf + htmlbuflen, "&gt;");
			htmlbuflen += 4;
		}
		else if(*str == '&')
		{
			if(!makeSpace(len + 4))
			{
				break;
			}
			strcpy(htmlbuf + htmlbuflen, "&amp;");
			htmlbuflen += 5;
		}
		else if(*str == '"')
		{
			if(!makeSpace(len + 4))
			{
				break;
			}
			strcpy(htmlbuf + htmlbuflen, "&quot;");
			htmlbuflen += 6;
		}
		else
		{
			htmlbuf[htmlbuflen++] = *str;
		}
		++str;
		--len;
	}
	htmlbuf[htmlbuflen] = '\0';
}

void NetPeekFormat::outputShort(const char *format, ...)
{
	va_list va;
	va_start(va, format);
	vsprintf(buffer + buflen, format, va);
	va_end(va);
	buflen += strlen(buffer + buflen);
}

void NetPeekFormat::outputShortString(const char *string, int len)
{
	if(len < 0)
	{
		strcpy(buffer + buflen, string);
		buflen += strlen(string);
	}
	else
	{
		memcpy(buffer + buflen, string, len);
		buflen += len;
	}
}

void NetPeekFormat::outputSection(const char *name)
{
	if(firstsection)
		firstsection = 0;
	else
		writeHTML("<p>");
	writeHTML("</font><font color=blue><b><u>");
	writeString(name);
	writeHTML("</u></b></font><br><font color=black>\n");
}

void NetPeekFormat::outputLong(const char *field, const char *format, ...)
{
	va_list va;
	char *temp;
	writeHTML("<font color=red>");
	writeString(field);
	writeHTML(": </font>");
	va_start(va, format);
	vsprintf(buffer, format, va);
	va_end(va);
	if((temp = strstr(buffer, "{{")) == 0)
	{
		writeString(buffer);
	}
	else
	{
		writeString(buffer, (int)(temp - buffer));
		writeHTML("&nbsp;&nbsp;&nbsp;");
		writeHTML("<font color=\"#008000\"><i>");
		writeString(temp + 2, strlen(temp) - 4);
		writeHTML("</i></font>");
	}
	writeHTML("<br>\n");
}

static char hexchars[] = "0123456789abcdef";

void NetPeekFormat::outputHex(const char *field, const void *data,
						      unsigned len)
{
	unsigned char *ptr = (unsigned char *)data;
	unsigned int addr;
	unsigned int offset;
	char *buf;
	char ch;
	if(field)
	{
		writeHTML("<font color=red>");
		writeString(field);
		writeHTML(": </font><br>\n");
	}
	addr = 0;
	writeHTML("<pre>");
	while(addr < len)
	{
		buf = buffer;
		for(offset = 0; offset < 16 && (addr + offset) < len; ++offset)
		{
			*buf++ = hexchars[(ptr[addr + offset] >> 4) & 0x0F];
			*buf++ = hexchars[ptr[addr + offset] & 0x0F];
			*buf++ = ' ';
		}
		while(offset < 16)
		{
			*buf++ = ' ';
			*buf++ = ' ';
			*buf++ = ' ';
			++offset;
		}
		*buf++ = ' ';
		for(offset = 0; offset < 16 && (addr + offset) < len; ++offset)
		{
			ch = ptr[addr + offset];
			if(ch == '<')
			{
				strcpy(buf, "&lt;");
				buf += 4;
			}
			else if(ch == '>')
			{
				strcpy(buf, "&gt;");
				buf += 4;
			}
			else if(ch == '&')
			{
				strcpy(buf, "&amp;");
				buf += 5;
			}
			else if(ch == '"')
			{
				strcpy(buf, "&quot;");
				buf += 6;
			}
			else if(ch >= 0x20 && ch <= 0x7E)
			{
				*buf++ = ch;
			}
			else
			{
				*buf++ = '.';
			}
		}
		strcpy(buf, "\n");
		writeHTML(buffer);
		addr += 16;
	}
	writeHTML("</pre>");
}

void NetPeekFormat::outputText(const char *field, const void *data,
						       unsigned len, int suppressCRs)
{
	char *ptr = (char *)data;
	char *buf;
	unsigned int buflen;
	int linelen;
	char ch;
	if(field)
	{
		writeHTML("<font color=red>");
		writeString(field);
		writeHTML(": </font><br>\n");
	}
	writeHTML("<pre>");
	buf = buffer;
	buflen = 0;
	linelen = 0;
	while(len-- > 0)
	{
		ch = *ptr++;
		if(ch >= 0x20 && ch <= 0x7E)
		{
			if(linelen >= 80)
			{
				*buf++ = '\\';
				*buf++ = '\n';
				buflen += 2;
				linelen = 0;
			}
			if(ch == '<')
			{
				strcpy(buf, "&lt;");
				buf += 4;
				buflen += 4;
				++linelen;
			}
			else if(ch == '>')
			{
				strcpy(buf, "&gt;");
				buf += 4;
				buflen += 4;
				++linelen;
			}
			else if(ch == '&')
			{
				strcpy(buf, "&amp;");
				buf += 5;
				buflen += 5;
				++linelen;
			}
			else if(ch == '"')
			{
				strcpy(buf, "&quot;");
				buf += 6;
				buflen += 6;
				++linelen;
			}
			else if(ch == '\\' || ch == '^')
			{
				*buf++ = '\\';
				*buf++ = (char)ch;
				buflen += 2;
				linelen += 2;
			}
			else
			{
				*buf++ = (char)ch;
				++buflen;
				++linelen;
			}
		}
		else if(ch == '\n')
		{
			*buf++ = '\n';
			++buflen;
			linelen = 0;
		}
		else if(ch == '\r')
		{
			if(!suppressCRs)
			{
				*buf++ = '^';
				*buf++ = 'M';
				buflen += 2;
				linelen += 2;
			}
		}
		else if(ch == '\t')
		{
			if(linelen >= 80)
			{
				*buf++ = '\\';
				*buf++ = '\n';
				buflen += 2;
				linelen = 0;
			}
			*buf++ = ' ';
			++buflen;
			++linelen;
			while((linelen % 8) != 0)
			{
				*buf++ = ' ';
				++buflen;
				++linelen;
			}
		}
		else if(ch > 0 && ch < 0x20)
		{
			if(linelen >= 80)
			{
				*buf++ = '\\';
				*buf++ = '\n';
				buflen += 2;
				linelen = 0;
			}
			*buf++ = '^';
			*buf++ = ch + '@';
			buflen += 2;
			linelen += 2;
		}
		else
		{
			if(linelen >= 80)
			{
				*buf++ = '\\';
				*buf++ = '\n';
				buflen += 2;
				linelen = 0;
			}
			*buf++ = '\\';
			*buf++ = '0' + (((int)ch) >> 6) & 0x03;
			*buf++ = '0' + (((int)ch) >> 3) & 0x07;
			*buf++ = '0' + ((int)ch) & 0x07;
			buflen += 4;
			linelen += 4;
		}
		if(buflen >= 2048)
		{
			*buf = '\0';
			writeHTML(buffer);
			buf = buffer;
			buflen = 0;
		}
	}
	if(buflen > 0)
	{
		*buf = '\0';
		writeHTML(buffer);
	}
	writeHTML("</pre>");
}

int NetPeekFormat::wantLinkHeader(void)
{
	return 0;
}

int NetPeekFormat::resolveIPAddresses(void)
{
	return pref_resolve_addresses;
}

int NetPeekFormat::fullyQualifiedNames(void)
{
	return pref_fully_qualified;
}

void NetPeekFormat::markIPAddress(const void *addr)
{
	if(numAddresses < NETPEEK_MAX_ADDRESSES)
	{
		memcpy(&(addresses[numAddresses]), addr, sizeof(bpf_u_int32));
		++numAddresses;
	}
}

void NetPeekFormat::shortFormColor(FormatColor _color)
{
	color = _color;
}

class NetPeekSink : public NetMonitorSink
{
public:

	NetPeekSink(FormatRuleTable *_rules);
	virtual ~NetPeekSink();

private:

	FormatRuleTable *rules;
	NetPeekFormat formatter;
	int32_t thiszone;
	int32_t today;
	int32_t prevday;
	char	datebuf[32];

public:

	FormatRuleTable *getRules() { return rules; }

	virtual void processPacket(NetPacket& packet);
	void displayPacket(NetPacket& packet);
	void updatePacket(NetPacket& packet, gint row);

};

NetPeekSink::NetPeekSink(FormatRuleTable *_rules)
{
	rules = _rules;
	thiszone = gmt2local(0);
	today = (time(0) + thiszone) / 86400;
	prevday = 0;
}

NetPeekSink::~NetPeekSink()
{
}

static void cb_destroy_packet_cache(gpointer data)
{
	free(data);
}

void NetPeekSink::processPacket(NetPacket& packet)
{
	// Allocate the packet display colors if necessary.
	if(!formatter.haveColors)
	{
		gdk_color_parse("#000000", &formatter.colors[(int)FormatColor_Normal]);
		gdk_color_parse("#008000", &formatter.colors[(int)FormatColor_UDP]);
		gdk_color_parse("#000000", &formatter.colors[(int)FormatColor_TCPData]);
		gdk_color_parse("#FF0000", &formatter.colors[(int)FormatColor_TCPConnect]);
		gdk_color_parse("#FF0000", &formatter.colors[(int)FormatColor_TCPDisconnect]);
		gdk_color_parse("#0000FF", &formatter.colors[(int)FormatColor_ICMP]);
		gdk_color_parse("#000000", &formatter.colors[(int)FormatColor_OtherIP]);
		gdk_color_parse("#008080", &formatter.colors[(int)FormatColor_ARP]);
		gdk_color_parse("#008080", &formatter.colors[(int)FormatColor_RARP]);
		gdk_color_parse("#008080", &formatter.colors[(int)FormatColor_OtherEthernet]);
		formatter.haveColors = 1;
	}

	// Format the packet into a buffer in short form.
	formatter.buflen = 0;
	formatter.color = FormatColor_Normal;
	formatter.addresses[0] = 0;
	formatter.addresses[1] = 0;
	formatter.numAddresses = 0;
	rules->formatPacketShort(packet, formatter, packet.linkType);
	formatter.buffer[formatter.buflen] = '\0';

	// Add the packet details to the short form list display in the UI.
	char numbuf[32];
	char timebuf[64];
	char sizebuf[32];
	gchar *info[4];
	unsigned long t = (packet.header->ts.tv_sec + thiszone);
	unsigned long day = t / 86400;
	t %= 86400;
	if(day != today)
	{
		// The data was captured on a different day, so include the date.
		if(day != prevday)
		{
			// Day changed since the last format call.
			struct tm *tms = localtime(&(packet.header->ts.tv_sec));
			prevday = day;
			memset(datebuf, 0, sizeof(datebuf));
			strftime(datebuf, sizeof(datebuf), "%x", tms);
		}
		sprintf(timebuf, "%s %02d:%02d:%02d", datebuf,
				(int)(t / 3600), (int)((t % 3600) / 60), (int)(t % 60));
	}
	else
	{
		// The data was captured today, so show the time only.
		sprintf(timebuf, "%02d:%02d:%02d.%06u",
				(int)(t / 3600), (int)((t % 3600) / 60),
				(int)(t % 60), (unsigned)(packet.header->ts.tv_usec));
	}
	sprintf(numbuf, "%lu", (unsigned long)(packet.packetNum));
	info[0] = numbuf;
	info[1] = timebuf;
	if(packet.linkType == DLT_EN10MB ||
	   packet.linkType == DLT_IEEE802)
	{
		// Include the Ethernet header in the size report.
		sprintf(sizebuf, "%d", (int)(packet.header->len));
	}
	else
	{
		sprintf(sizebuf, "%d",
				(int)(packet.header->len - packet.linkAddrLen));
	}
	info[2] = sizebuf;
	info[3] = formatter.buffer;
	gint index = gtk_clist_append(GTK_CLIST(app_short_form), info);
	gtk_clist_set_row_data_full(GTK_CLIST(app_short_form), index,
							packet.convertIntoCached(formatter.addresses[0],
													 formatter.addresses[1]),
							cb_destroy_packet_cache);
	gtk_clist_set_foreground(GTK_CLIST(app_short_form), index,
							 &formatter.colors[(int)formatter.color]);
	if(pref_auto_scroll && !(global_net_monitor.isCaptureFile()))
	{
		gtk_clist_moveto(GTK_CLIST(app_short_form), index, 0, 0.0, 1.0);
	}

	// Increment and display the received packet count.
	++global_net_packet_count;
	sprintf(formatter.buffer, _("%lu packets received"),
			global_net_packet_count);
	gnome_appbar_set_default(GNOME_APPBAR(app_status_bar), formatter.buffer);
	if(global_net_packet_count == 1 && !global_net_monitor.isCaptureFile())
	{
		updateMenus();
	}
}

void NetPeekSink::displayPacket(NetPacket& packet)
{
	// Convert the packet contents into a HTML data stream.
	formatter.clearHTML();
	formatter.writeHTML("<body bgcolor=\"#ffffff\"><font color=black>\n");
	if(pref_hex_packets)
	{
#ifdef linux
		if(packet.linkType == DLT_NULL)
		{
			// The NULL link type puts 4 bytes of garbage on the front of
			// packets under Linux, which the user won't be interested in.
			formatter.outputHex(0, packet.payload, packet.payloadLen);
		}
		else
#endif
		{
			formatter.outputHex(0, packet.subPacket, packet.subPacketLen);
		}
	}
	else
	{
		rules->formatPacketLong(packet, formatter, packet.linkType);
	}
	formatter.writeHTML("</font></body>");

	// Display the HTML to the user.
	gtk_xmhtml_source(GTK_XMHTML(app_long_form), formatter.htmlbuf);
}

void NetPeekSink::updatePacket(NetPacket& packet, gint row)
{
	// Format the packet into a buffer in short form.
	formatter.buflen = 0;
	formatter.color = FormatColor_Normal;
	formatter.addresses[0] = 0;
	formatter.addresses[1] = 0;
	formatter.numAddresses = 0;
	rules->formatPacketShort(packet, formatter, packet.linkType);
	formatter.buffer[formatter.buflen] = '\0';

	// Update the column list display to reflect the new formatted form.
	gtk_clist_set_text(GTK_CLIST(app_short_form), row, 3, formatter.buffer);
}

static void cb_file_open(GtkWidget *widget);
static void cb_file_save(GtkWidget *widget);
static void cb_file_exit(GtkWidget *widget);
static void cb_file_print(GtkWidget *widget);

static void cb_capture_start(GtkWidget *widget);
static void cb_capture_stop(GtkWidget *widget);
static void cb_capture_clear(GtkWidget *widget);

static void cb_tools_ping(GtkWidget *widget);
static void cb_tools_traceroute(GtkWidget *widget);
static void cb_tools_host_lookup(GtkWidget *widget);
static void cb_tools_network_interfaces(GtkWidget *widget);
static void cb_tools_tcp_sockets(GtkWidget *widget);
static void cb_tools_udp_sockets(GtkWidget *widget);
static void cb_tools_unix_sockets(GtkWidget *widget);
static void cb_tools_routing_table(GtkWidget *widget);
static void cb_tools_arp_table(GtkWidget *widget);
static void cb_tools_rpc_services(GtkWidget *widget);

static void cb_settings_filter(GtkWidget *widget);
static void cb_settings_prefs(GtkWidget *widget);

static void cb_help_about(GtkWidget *widget);

static gboolean cb_file_open_enabled(void *data);
static gboolean cb_file_save_enabled(void *data);
static gboolean cb_file_print_enabled(void *data);
static gboolean cb_capture_start_enabled(void *data);
static gboolean cb_capture_stop_enabled(void *data);
static gboolean cb_capture_clear_enabled(void *data);
static gboolean cb_tools_ping_enabled(GtkWidget *widget);
static gboolean cb_tools_traceroute_enabled(GtkWidget *widget);
static gboolean cb_tools_host_lookup_enabled(GtkWidget *widget);
static gboolean cb_tools_network_interfaces_enabled(GtkWidget *widget);
static gboolean cb_tools_tcp_sockets_enabled(GtkWidget *widget);
static gboolean cb_tools_udp_sockets_enabled(GtkWidget *widget);
static gboolean cb_tools_unix_sockets_enabled(GtkWidget *widget);
static gboolean cb_tools_routing_table_enabled(GtkWidget *widget);
static gboolean cb_tools_arp_table_enabled(GtkWidget *widget);
static gboolean cb_tools_rpc_services_enabled(GtkWidget *widget);
static gboolean cb_settings_filter_enabled(void *data);

static GnomeUIInfo file_menu[] = {
	GNOMEUIINFO_ITEM_STOCK(N_("_Open"),
						   N_("Open a packet trace file"),
						   cb_file_open, GNOME_STOCK_MENU_OPEN),
	GNOMEUIINFO_ITEM_STOCK(N_("_Save"),
						   N_("Save the current packet trace to a file"),
						   cb_file_save, GNOME_STOCK_MENU_SAVE),
#if 0
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_STOCK(N_("_Print"),
						   N_("Print the current packet trace"),
						   cb_file_print, GNOME_STOCK_MENU_PRINT),
#endif
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_EXIT_ITEM(cb_file_exit, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo capture_menu[] = {
	GNOMEUIINFO_ITEM_STOCK(N_("_Start"),
						   N_("Start capturing network packets"),
						   cb_capture_start,
						   "Menu_StartCapture"),
	GNOMEUIINFO_ITEM_STOCK(N_("S_top"),
						   N_("Stop capturing network packets"),
						   cb_capture_stop,
						   "Menu_StopCapture"),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_NONE(N_("_Clear"),
						  N_("Clear the packet trace"),
						  cb_capture_clear),
};

static GnomeUIInfo settings_menu[] = {
	GNOMEUIINFO_MENU_PREFERENCES_ITEM(cb_settings_prefs, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_NONE(N_("_Filtering conditions..."),
						  N_("Set packet filtering conditions"),
						  cb_settings_filter),
	GNOMEUIINFO_END
};

static GnomeUIInfo tools_menu[] = {
	GNOMEUIINFO_ITEM_NONE(N_("_Ping..."),
						  N_("Ping a host to see if it is alive"),
						  cb_tools_ping),
#if 0
	GNOMEUIINFO_ITEM_NONE(N_("_Traceroute..."),
						  N_("Trace the network route to a host"),
						  cb_tools_traceroute),
#endif
	GNOMEUIINFO_ITEM_NONE(N_("_Host lookup..."),
						  N_("Look up hosts and IP addresses using DNS"),
						  cb_tools_host_lookup),
	GNOMEUIINFO_ITEM_NONE(N_("_Network interfaces..."),
						  N_("Show the network interfaces on this host"),
						  cb_tools_network_interfaces),
	GNOMEUIINFO_ITEM_NONE(N_("Active T_CP sockets..."),
						  N_("Show the active TCP sockets on this host"),
						  cb_tools_tcp_sockets),
	GNOMEUIINFO_ITEM_NONE(N_("Active _UDP sockets..."),
						  N_("Show the active UDP sockets on this host"),
						  cb_tools_udp_sockets),
	GNOMEUIINFO_ITEM_NONE(N_("Active Unix _domain sockets..."),
				  N_("Show the active Unix domain sockets on this host"),
						  cb_tools_unix_sockets),
	GNOMEUIINFO_ITEM_NONE(N_("_Routing table..."),
						  N_("Show the routing table on this host"),
						  cb_tools_routing_table),
	GNOMEUIINFO_ITEM_NONE(N_("_ARP table..."),
						  N_("Show the ARP table on this host"),
						  cb_tools_arp_table),
	GNOMEUIINFO_ITEM_NONE(N_("RPC _services..."),
				  N_("Show the RPC services that are running on this host"),
				  cb_tools_rpc_services),
	GNOMEUIINFO_END
};

static GnomeUIInfo help_menu[] = {
	/*GNOMEUIINFO_HELP(NETPEEK_NAME_CONFIG),*/
	GNOMEUIINFO_MENU_ABOUT_ITEM(cb_help_about, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo menubar[] = {
	GNOMEUIINFO_MENU_FILE_TREE(file_menu),
	GNOMEUIINFO_SUBTREE(N_("_Capture"), capture_menu),
	GNOMEUIINFO_SUBTREE(N_("_Tools"), tools_menu),
	GNOMEUIINFO_MENU_SETTINGS_TREE(settings_menu),
	GNOMEUIINFO_MENU_HELP_TREE(help_menu),
	GNOMEUIINFO_END
};

static GnomeUIInfo toolbar[] = {
	GNOMEUIINFO_ITEM_STOCK(N_("Open"),
						   N_("Open a packet trace file"),
						   cb_file_open, GNOME_STOCK_PIXMAP_OPEN),
	GNOMEUIINFO_ITEM_STOCK(N_("Save"),
						   N_("Save the current packet trace to a file"),
						   cb_file_save, GNOME_STOCK_PIXMAP_SAVE),
	GNOMEUIINFO_SEPARATOR,
#if 0
	GNOMEUIINFO_ITEM_STOCK(N_("Print"),
						   N_("Print the current packet trace"),
						   cb_file_print, GNOME_STOCK_PIXMAP_PRINT),
	GNOMEUIINFO_SEPARATOR,
#endif
	GNOMEUIINFO_ITEM_STOCK(N_("Start"),
						   N_("Start capturing network packets"),
						   cb_capture_start, "Toolbar_StartCapture"),
	GNOMEUIINFO_ITEM_STOCK(N_("Stop"),
						   N_("Stop capturing network packets"),
						   cb_capture_stop, "Toolbar_StopCapture"),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_STOCK(N_("Filter"),
						   N_("Set packet filtering conditions"),
						   cb_settings_filter, GNOME_STOCK_PIXMAP_HELP),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_STOCK(N_("Clear"),
						   N_("Clear the packet trace"),
						   cb_capture_clear, GNOME_STOCK_PIXMAP_CLEAR),
	GNOMEUIINFO_END
};

/*
 * Menu state mapping information.
 */
static MenuStateInfo menu_states[] = {
	MENUSTATEINFO("File/Open", cb_file_open_enabled, NULL),
	MENUSTATEINFO("File/Save", cb_file_save_enabled, NULL),
#if 0
	MENUSTATEINFO("File/Print", cb_file_print_enabled, NULL),
#endif
	MENUSTATEINFO("Capture/Start", cb_capture_start_enabled, NULL),
	MENUSTATEINFO("Capture/Stop", cb_capture_stop_enabled, NULL),
	MENUSTATEINFO("Capture/Clear", cb_capture_clear_enabled, NULL),
	MENUSTATEINFO("Tools/Ping...", cb_tools_ping_enabled, NULL),
#if 0
	MENUSTATEINFO("Tools/Traceroute...", cb_tools_traceroute_enabled, NULL),
#endif
	MENUSTATEINFO("Tools/Host lookup...", cb_tools_host_lookup_enabled, NULL),
	MENUSTATEINFO("Tools/Network interfaces...",
				  cb_tools_network_interfaces_enabled, NULL),
	MENUSTATEINFO("Tools/Active TCP sockets...",
				  cb_tools_tcp_sockets_enabled, NULL),
	MENUSTATEINFO("Tools/Active UDP sockets...",
				  cb_tools_udp_sockets_enabled, NULL),
	MENUSTATEINFO("Tools/Active Unix domain sockets...",
				  cb_tools_unix_sockets_enabled, NULL),
	MENUSTATEINFO("Tools/Routing table...",
				  cb_tools_routing_table_enabled, NULL),
	MENUSTATEINFO("Tools/ARP table...",
				  cb_tools_arp_table_enabled, NULL),
	MENUSTATEINFO("Tools/RPC services...",
				  cb_tools_rpc_services_enabled, NULL),
	MENUSTATEINFO("Settings/Filtering conditions...",
				  cb_settings_filter_enabled, NULL),
	MENUSTATEINFO_END
};

/*
 * Toolbar mapping information, which maps menu items to
 * toolbar indexes, to assist in greying out buttons when
 * certain actions aren't possible.
 */
static ToolbarMappingInfo toolbar_mappings[] = {
	TOOLBARMAPPINGINFO("File/Open", 0),
	TOOLBARMAPPINGINFO("File/Save", 1),
#if 0
	TOOLBARMAPPINGINFO("File/Print", 3),
#endif
	TOOLBARMAPPINGINFO("Capture/Start", 3),
	TOOLBARMAPPINGINFO("Capture/Stop", 4),
	TOOLBARMAPPINGINFO("Settings/Filtering conditions...", 6),
	TOOLBARMAPPINGINFO("Capture/Clear", 8),
	TOOLBARMAPPINGINFO_END
};

#define	COL_CHARS(n)		((n) * 8)

static gchar *columnTitles[] =
	{N_("Pkt #"), N_("Time Stamp"), N_("Size"), N_("Details")};
#define	num_columns	4
static gint columnWidths[] =
	{COL_CHARS(4), COL_CHARS(13), COL_CHARS(4)};

static void cb_monitor_data(gpointer data, gint source, GdkInputCondition cond)
{
	global_net_monitor.processSelect();
}

static void cb_select_packet(GtkCList *clist, gint row, gint column,
							 GdkEvent *event, gpointer data)
{
	NetPacketCached *cache =
		(NetPacketCached *)gtk_clist_get_row_data(clist, row);
	if(global_net_sink && cache)
	{
		NetPacket packet(cache->linkType, &(cache->header), cache->packet,
						 cache->packetNum);
		packet.parseBasic();
		global_net_sink->displayPacket(packet);
		app_short_form_selected = row;
	}
	else
	{
		gtk_xmhtml_source(GTK_XMHTML(app_long_form),
						  "<body bgcolor=\"#ffffff\"></body>");
		app_short_form_selected = -1;
	}
}

static char *find_program(char *name)
{
	char pathname[1024];
	sprintf(pathname, "/sbin/%s", name);
	if(access(pathname, X_OK) >= 0)
	{
		return g_strdup(pathname);
	}
	sprintf(pathname, "/usr/sbin/%s", name);
	if(access(pathname, X_OK) >= 0)
	{
		return g_strdup(pathname);
	}
	sprintf(pathname, "/bin/%s", name);
	if(access(pathname, X_OK) >= 0)
	{
		return g_strdup(pathname);
	}
	sprintf(pathname, "/usr/bin/%s", name);
	if(access(pathname, X_OK) >= 0)
	{
		return g_strdup(pathname);
	}
	sprintf(pathname, "/usr/ucb/%s", name);
	if(access(pathname, X_OK) >= 0)
	{
		return g_strdup(pathname);
	}
	sprintf(pathname, "/usr/local/bin/%s", name);
	if(access(pathname, X_OK) >= 0)
	{
		return g_strdup(pathname);
	}
	sprintf(pathname, "/opt/local/bin/%s", name);
	if(access(pathname, X_OK) >= 0)
	{
		return g_strdup(pathname);
	}
	sprintf(pathname, "/opt/bin/%s", name);
	if(access(pathname, X_OK) >= 0)
	{
		return g_strdup(pathname);
	}
	return gnome_is_program_in_path(name);
}

static gint app_resolver_select_id = -1;
static guint app_resolver_idle_id = (guint)(-1);
static struct in_addr *resolver_addresses = 0;
static gint num_resolver_addresses = 0;
static gint max_resolver_addresses = 0;

static gboolean ResolverIdleProc(gpointer data)
{
	GtkCList *clist;
	gint index, numrows, addr;
	NetPacketCached *packet;
	gboolean needformat;

	// Update the displayed packet list wherever an address
	// in our resolver list is encountered.
	clist = GTK_CLIST(app_short_form);
	numrows = clist->rows;
	gtk_clist_freeze(clist);
	for(index = 0; index < numrows; ++index)
	{
		packet = (NetPacketCached *)(gtk_clist_get_row_data(clist, index));
		if(packet && (packet->addr1 || packet->addr2))
		{
			needformat = FALSE;
			for(addr = 0; addr < num_resolver_addresses; ++addr)
			{
				if(packet->addr1 == resolver_addresses[addr].s_addr)
				{
					packet->addr1 = 0;
					needformat = TRUE;
					if(!(packet->addr2))
					{
						break;
					}
				}
				if(packet->addr2 == resolver_addresses[addr].s_addr)
				{
					packet->addr2 = 0;
					needformat = TRUE;
					if(!(packet->addr1))
					{
						break;
					}
				}
			}
			if(needformat)
			{
				// We've found a match, so we need to reformat this packet.
				NetPacket netpacket(packet->linkType, &(packet->header),
									packet->packet, packet->packetNum);
				netpacket.parseBasic();
				global_net_sink->updatePacket(netpacket, index);
			}
		}
	}
	gtk_clist_thaw(clist);

	// Reset the address list.
	num_resolver_addresses = 0;

	// Remove the idle handler.
	app_resolver_idle_id = (guint)(-1);
	return FALSE;
}

void ResolverCallback(struct in_addr address)
{
	// Add the new address to the list.
	if(num_resolver_addresses >= max_resolver_addresses)
	{
		struct in_addr *newlist;
		newlist = (struct in_addr *)realloc(resolver_addresses,
					sizeof(struct in_addr *) * (max_resolver_addresses + 16));
		if(!newlist)
		{
			return;
		}
		resolver_addresses = newlist;
		max_resolver_addresses += 16;
	}
	resolver_addresses[num_resolver_addresses++] = address;

	// Install the idle activity callback if not already done.
	if(app_resolver_idle_id == (guint)(-1))
	{
		app_resolver_idle_id = gtk_idle_add(ResolverIdleProc, NULL);
	}
}

void ResolverDisappeared(int fd)
{
	if(app_resolver_select_id != -1)
	{
		gdk_input_remove(app_resolver_select_id);
		app_resolver_select_id = -1;
	}
}

static void ResolverSelectProc(gpointer data, gint source,
							   GdkInputCondition cond)
{
	ResolverSelect();
}

int main(int argc, char *argv[])
{
	char errbuf[PCAP_ERRBUF_SIZE];
	FormatRuleTable *rules;
	gint col;
	int resolver_fd;

	// Start the IP address to hostname resolver in a separate process.
	resolver_fd = ResolverStart(argv[0]);

	// Create the rule list to use to format packets.
	rules = FormatRulesCreate();

	// Initialize GNOME.
	gnome_init(NETPEEK_NAME_CONFIG, NETPEEK_NAME_VERSION, argc, argv);

	// Initialize the session manager.
	app_client = gnome_master_client();

	// Load the preference data.
	pref_resolve_addresses = gnome_config_get_bool
			("/" NETPEEK_NAME_CONFIG "/preferences/resolve_addresses=true");
	pref_resolve_foreign = gnome_config_get_bool
			("/" NETPEEK_NAME_CONFIG "/preferences/resolve_foreign=true");
	pref_fully_qualified = gnome_config_get_bool
			("/" NETPEEK_NAME_CONFIG "/preferences/fully_qualified=true");
	pref_hex_packets = gnome_config_get_bool
			("/" NETPEEK_NAME_CONFIG "/preferences/hex_packets=false");
	pref_capture_length = gnome_config_get_int
			("/" NETPEEK_NAME_CONFIG "/preferences/capture_length=2048");
	pref_auto_scroll = gnome_config_get_bool
			("/" NETPEEK_NAME_CONFIG "/preferences/auto_scroll=true");

	// Load the filter data.
	global_filter.addrType = (FilterAddressType)gnome_config_get_int
		("/" NETPEEK_NAME_CONFIG "/filter/address_type=0");
	global_filter.setAddrTo(gnome_config_get_string
		("/" NETPEEK_NAME_CONFIG "/filter/address_to="));
	global_filter.setAddrFrom(gnome_config_get_string
		("/" NETPEEK_NAME_CONFIG "/filter/address_from="));
	global_filter.setAddrToOrFrom(gnome_config_get_string
		("/" NETPEEK_NAME_CONFIG "/filter/address_to_or_from="));
	global_filter.setAddrBetween(gnome_config_get_string
		("/" NETPEEK_NAME_CONFIG "/filter/address_between="));
	global_filter.setAddrAnd(gnome_config_get_string
		("/" NETPEEK_NAME_CONFIG "/filter/address_and="));
	global_filter.protocols = (FilterProtocolType)gnome_config_get_int
		("/" NETPEEK_NAME_CONFIG "/filter/protocols=127");
	global_filter.portEnabled = gnome_config_get_bool
		("/" NETPEEK_NAME_CONFIG "/filter/port_enabled=false");
	global_filter.setPorts(gnome_config_get_string
		("/" NETPEEK_NAME_CONFIG "/filter/ports="));

	// Find system programs that we are interested in.
	netstat_program = find_program("netstat");
	arp_program = find_program("arp");
	ifconfig_program = find_program("ifconfig");
	rpcinfo_program = find_program("rpcinfo");

	// Register extra stock pixmaps for use in menus and toolbars.
	gnome_stock_pixmap_register("Toolbar_StartCapture",
					GNOME_STOCK_PIXMAP_REGULAR,
					(GnomeStockPixmapEntry *)&capture_start_regular);
	gnome_stock_pixmap_register("Toolbar_StartCapture",
					GNOME_STOCK_PIXMAP_DISABLED,
					(GnomeStockPixmapEntry *)&capture_start_disabled);
	gnome_stock_pixmap_register("Toolbar_StartCapture",
					GNOME_STOCK_PIXMAP_FOCUSED,
					(GnomeStockPixmapEntry *)&capture_start_focused);
	gnome_stock_pixmap_register("Toolbar_StopCapture",
					GNOME_STOCK_PIXMAP_REGULAR,
					(GnomeStockPixmapEntry *)&capture_stop_regular);
	gnome_stock_pixmap_register("Toolbar_StopCapture",
					GNOME_STOCK_PIXMAP_DISABLED,
					(GnomeStockPixmapEntry *)&capture_stop_disabled);
	gnome_stock_pixmap_register("Toolbar_StopCapture",
					GNOME_STOCK_PIXMAP_FOCUSED,
					(GnomeStockPixmapEntry *)&capture_stop_focused);
	gnome_stock_pixmap_register("Menu_StartCapture",
					GNOME_STOCK_PIXMAP_REGULAR,
					(GnomeStockPixmapEntry *)&capture_start_small_regular);
	gnome_stock_pixmap_register("Menu_StartCapture",
					GNOME_STOCK_PIXMAP_DISABLED,
					(GnomeStockPixmapEntry *)&capture_start_small_disabled);
	gnome_stock_pixmap_register("Menu_StartCapture",
					GNOME_STOCK_PIXMAP_FOCUSED,
					(GnomeStockPixmapEntry *)&capture_start_small_focused);
	gnome_stock_pixmap_register("Menu_StopCapture",
					GNOME_STOCK_PIXMAP_REGULAR,
					(GnomeStockPixmapEntry *)&capture_stop_small_regular);
	gnome_stock_pixmap_register("Menu_StopCapture",
					GNOME_STOCK_PIXMAP_DISABLED,
					(GnomeStockPixmapEntry *)&capture_stop_small_disabled);
	gnome_stock_pixmap_register("Menu_StopCapture",
					GNOME_STOCK_PIXMAP_FOCUSED,
					(GnomeStockPixmapEntry *)&capture_stop_small_focused);

	// Create the top-level application window.
	app_main_window = GNOME_APP(gnome_app_new(NETPEEK_NAME_CONFIG,
									          NETPEEK_NAME_DISPLAY));
	gtk_window_set_default_size(GTK_WINDOW(app_main_window), 600, 450);
	gnome_app_create_menus(app_main_window, menubar);
	gnome_app_create_toolbar(app_main_window, toolbar);
	app_status_bar = GNOME_APPBAR(gnome_appbar_new(FALSE, TRUE,
												   GNOME_PREFERENCES_NEVER));
	gnome_app_set_statusbar(app_main_window, GTK_WIDGET(app_status_bar));
	gnome_app_install_appbar_menu_hints(app_status_bar, menubar);

	// Create the contents area for the top-level window.
	GtkWidget *scroller = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroller),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC);
	app_short_form = gtk_clist_new_with_titles(num_columns, columnTitles);
	gtk_container_add(GTK_CONTAINER(scroller), app_short_form);
	gtk_clist_column_titles_passive(GTK_CLIST(app_short_form));
	for(col = 0; col < num_columns - 1; ++col)
	{
		gtk_clist_set_column_width(GTK_CLIST(app_short_form), col,
								   columnWidths[col]);
	}
	gtk_clist_set_column_justification(GTK_CLIST(app_short_form), 0,
									   GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification(GTK_CLIST(app_short_form), 2,
									   GTK_JUSTIFY_RIGHT);
	app_long_form = gtk_xmhtml_new();
	GTK_XMHTML(app_long_form)->html.margin_width = 5;
	GTK_XMHTML(app_long_form)->html.margin_height = 5;
	GtkWidget *frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(frame), app_long_form);
	app_contents = gtk_vpaned_new();
	gtk_paned_pack1(GTK_PANED(app_contents), scroller/*app_short_form*/, FALSE, TRUE);
	gtk_paned_pack2(GTK_PANED(app_contents), frame, TRUE, TRUE);
	gtk_paned_set_position(GTK_PANED(app_contents), 120);
	gnome_app_set_contents(app_main_window, app_contents);
	gtk_xmhtml_source(GTK_XMHTML(app_long_form),
					  "<body bgcolor=\"#ffffff\"></body>");
	gtk_widget_show(app_short_form);
	gtk_widget_show(frame);
	gtk_widget_show(app_long_form);
	gtk_widget_show(scroller);

	// Add a monitor sink to receive incoming packets during packet captures.
	global_net_sink = new NetPeekSink(rules);
	global_net_monitor.addMonitorSink(global_net_sink);

	// Trap interesting signals.
	gtk_signal_connect(GTK_OBJECT(app_main_window), "destroy",
					   GTK_SIGNAL_FUNC(cb_file_exit), NULL);
	gtk_signal_connect(GTK_OBJECT(app_short_form), "select_row",
					   GTK_SIGNAL_FUNC(cb_select_packet), NULL);

	// Set the initial menu and toolbar item states.
	menu_util_process_items(app_main_window, menu_states, toolbar_mappings, 0);

	// Display the main application window.
	gtk_widget_show(GTK_WIDGET(app_main_window));

	// Start selecting on the resolver's file descriptor.
	if(resolver_fd != -1)
	{
		app_resolver_select_id = gdk_input_add(resolver_fd,
					  					       GDK_INPUT_READ,
											   ResolverSelectProc,
											   NULL);
	}

	// Enter the main application loop.
	gtk_main();
	return 0;
}

static void clearWindows(void)
{
	gtk_clist_clear(GTK_CLIST(app_short_form));
	gtk_xmhtml_source(GTK_XMHTML(app_long_form),
					  "<body bgcolor=\"#ffffff\"></body>");
	global_net_packet_count = 0;
	app_short_form_selected = -1;
}

static void updateMenus(void)
{
	menu_util_process_items(app_main_window, menu_states, toolbar_mappings, 0);
}

static void cb_cancel_dialog(GtkWidget *widget, GtkWidget **toclose)
{
	gtk_widget_destroy(*toclose);
	*toclose = NULL;
}

static void setResolverInfo(char *device)
{
	struct in_addr address;
	struct in_addr netmask;
	char errbuf[PCAP_ERRBUF_SIZE];
	if(!device || pcap_lookupnet(device, &(address.s_addr),
							     &(netmask.s_addr), errbuf) != 0)
	{
		// Use the loopback as our default network if we can't
		// find something else more reasonable to use.
		address.s_addr = htonl(0x7F000001);
		netmask.s_addr = htonl(0xFF000000);
	}
	ResolverNetwork(address, netmask, pref_resolve_foreign);
}

static void cb_open_dialog_ok(GtkWidget *widget)
{
	gchar *message;
	char errbuf[PCAP_ERRBUF_SIZE];

	if(global_net_monitor.openFile(gtk_file_selection_get_filename(GTK_FILE_SELECTION(app_open_dialog)), errbuf))
	{
		// "Capture" all packets in the file and then close it.
		gtk_clist_freeze(GTK_CLIST(app_short_form));
		clearWindows();
		global_net_monitor.processSelect();
		global_net_monitor.closeCapture();
		gtk_clist_thaw(GTK_CLIST(app_short_form));

		// Report that the file has been loaded.
		message = g_strdup_printf(_("Loaded file %s, %lu packets read"),
			gtk_file_selection_get_filename
					(GTK_FILE_SELECTION(app_open_dialog)),
			global_net_packet_count);
		gnome_appbar_set_default(GNOME_APPBAR(app_status_bar), message);
		g_free(message);
	}
	else
	{
		if(errbuf[0] == '/')
		{
			// The pcap library already put the filename in the error for us.
			message = g_strdup_printf(_("Can not open %s"), errbuf);
		}
		else
		{
			// We need to put the filename in ourselves.
			message = g_strdup_printf(_("Can not open %s: %s"),
				gtk_file_selection_get_filename
						(GTK_FILE_SELECTION(app_open_dialog)),
			  	errbuf);
		}
		gnome_error_dialog_parented(message, GTK_WINDOW(app_main_window));
		g_free(message);
	}

	// Set the network interface information for the resolver
	// to that of the default network interface.
	setResolverInfo(pcap_lookupdev(errbuf));

	gtk_widget_destroy(app_open_dialog);
	app_open_dialog = NULL;

	// Update the menu and toolbar states.
	menu_util_process_items(app_main_window, menu_states, toolbar_mappings, 0);
}

static void cb_file_open(GtkWidget *widget)
{
	// Stop an existing network packet capture, if any.
	if(app_input_select_fd != -1)
	{
		cb_capture_stop(widget);
	}

	// If we don't have an open dialog yet, then create one.
	if(!app_open_dialog)
	{
		app_open_dialog = gtk_file_selection_new(_("Open packet trace"));
	}

	// Position the open dialog and trap interesting signals.
	gtk_window_position(GTK_WINDOW(app_open_dialog), GTK_WIN_POS_MOUSE);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(app_open_dialog)
							->ok_button),
					   "clicked", GTK_SIGNAL_FUNC(cb_open_dialog_ok), NULL);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(app_open_dialog)
							->cancel_button),
					   "clicked", GTK_SIGNAL_FUNC(cb_cancel_dialog),
					   &(app_open_dialog));

	// Show the open dialog.
	gtk_widget_show(app_open_dialog);
}

static void cb_save_dialog_ok(GtkWidget *widget)
{
	gchar *filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(app_save_dialog));
	pcap_t temppcap;
	pcap_dumper_t *dumper;
	gchar *message;

	// Fake up a pcap structure that can be passed to pcap_dump_open
	// to write out the correct trace file header.
	temppcap.linktype = global_net_monitor.getLinkType();
	temppcap.tzoff = gmt2local(0);
	temppcap.snapshot = global_net_monitor.getSnapLen();

	// If the file already exists, then we need to ask the user if
	// they really want to overwrite it.
	if(access(filename, 0) >= 0)
	{
		message = g_strdup_printf(_("%s already exists.  Overwrite it?"),
								  filename);
		GtkWidget *askdialog =
			gnome_ok_cancel_dialog_modal_parented(message, NULL, NULL,
												  GTK_WINDOW(app_main_window));
		g_free(message);
		gint button = gnome_dialog_run(GNOME_DIALOG(askdialog));
		if(button != 0)
		{
			goto dontsave;
		}
	}

	// Attempt to open the dump file.
	if((dumper = pcap_dump_open(&temppcap, filename)) != NULL)
	{
		// Dump all packets to the file.
		unsigned long count = 0;
		gint index;
		NetPacketCached *cache;
		for(index = 0; index < GTK_CLIST(app_short_form)->rows; ++index)
		{
			cache = (NetPacketCached *)(gtk_clist_get_row_data(GTK_CLIST(app_short_form), index));
			if(cache)
			{
				pcap_dump((u_char *)dumper, &(cache->header), cache->packet);
				++count;
			}
		}
		pcap_dump_close(dumper);

		// Report the number of packets that were written.
		message = g_strdup_printf(_("Wrote %lu packets to %s"),
								  count, filename);
		gnome_appbar_set_default(GNOME_APPBAR(app_status_bar), message);
		g_free(message);
	}
	else
	{
		message = g_strdup_printf(_("Can not save to %s: %s"), filename,
								  g_strerror(errno));
		gnome_error_dialog_parented(message, GTK_WINDOW(app_main_window));
		g_free(message);
	}

dontsave:
	gtk_widget_destroy(app_save_dialog);
	app_save_dialog = NULL;

	// Update the menu and toolbar states.
	menu_util_process_items(app_main_window, menu_states, toolbar_mappings, 0);
}

static void cb_file_save(GtkWidget *widget)
{
	// If we don't have a save dialog yet, then create one.
	if(!app_save_dialog)
	{
		app_save_dialog = gtk_file_selection_new(_("Save packet trace"));
	}

	// Position the save dialog and trap interesting signals.
	gtk_window_position(GTK_WINDOW(app_save_dialog), GTK_WIN_POS_MOUSE);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(app_save_dialog)
							->ok_button),
					   "clicked", GTK_SIGNAL_FUNC(cb_save_dialog_ok), NULL);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(app_save_dialog)
							->cancel_button),
					   "clicked", GTK_SIGNAL_FUNC(cb_cancel_dialog),
					   &(app_save_dialog));

	// Show the save dialog.
	gtk_widget_show(app_save_dialog);
}

static void cb_file_print(GtkWidget *widget)
{
	int index, rows;
	NetPacketCached *cache;
	rows = GTK_CLIST(app_short_form)->rows;
	PrintFormatOutput printer(global_net_sink->getRules(), rows);
	if(!printer.openFile("netpeek.txt"))
	{
		return;
	}
	for(index = 0; index < rows; ++index)
	{
		cache = (NetPacketCached *)gtk_clist_get_row_data
								(GTK_CLIST(app_short_form), index);
		if(cache)
		{
			NetPacket packet(cache->linkType, &(cache->header),
							 cache->packet, cache->packetNum);
			packet.parseBasic();
			printer.writeShort(packet);
			//printer.writeLong(packet);
		}
	}
	printer.close();
}

static void cb_file_exit(GtkWidget *widget)
{
	// Save the current preferences.
	gnome_config_set_bool
		("/" NETPEEK_NAME_CONFIG "/preferences/resolve_addresses",
		 pref_resolve_addresses);
	gnome_config_set_bool
		("/" NETPEEK_NAME_CONFIG "/preferences/resolve_foreign",
		 pref_resolve_foreign);
	gnome_config_set_bool
		("/" NETPEEK_NAME_CONFIG "/preferences/fully_qualified",
		 pref_fully_qualified);
	gnome_config_set_bool
		("/" NETPEEK_NAME_CONFIG "/preferences/hex_packets",
		 pref_hex_packets);
	gnome_config_set_int
		("/" NETPEEK_NAME_CONFIG "/preferences/capture_length",
		 pref_capture_length);
	gnome_config_set_bool
		("/" NETPEEK_NAME_CONFIG "/preferences/auto_scroll",
		 pref_auto_scroll);

	// Save the current filtering conditions.
	gnome_config_set_int
		("/" NETPEEK_NAME_CONFIG "/filter/address_type",
		 (int)(global_filter.addrType));
	gnome_config_set_string
		("/" NETPEEK_NAME_CONFIG "/filter/address_to",
		 (global_filter.addrTo ? global_filter.addrTo : ""));
	gnome_config_set_string
		("/" NETPEEK_NAME_CONFIG "/filter/address_from",
		 (global_filter.addrFrom ? global_filter.addrFrom : ""));
	gnome_config_set_string
		("/" NETPEEK_NAME_CONFIG "/filter/address_to_or_from",
		 (global_filter.addrToOrFrom ? global_filter.addrToOrFrom : ""));
	gnome_config_set_string
		("/" NETPEEK_NAME_CONFIG "/filter/address_between",
		 (global_filter.addrBetween ? global_filter.addrBetween : ""));
	gnome_config_set_string
		("/" NETPEEK_NAME_CONFIG "/filter/address_and",
		 (global_filter.addrAnd ? global_filter.addrAnd : ""));
	gnome_config_set_int
		("/" NETPEEK_NAME_CONFIG "/filter/protocols",
		 (int)(global_filter.protocols));
	gnome_config_set_bool
		("/" NETPEEK_NAME_CONFIG "/filter/port_enabled",
		global_filter.portEnabled);
	gnome_config_set_string
		("/" NETPEEK_NAME_CONFIG "/filter/ports",
		 (global_filter.ports ? global_filter.ports : ""));

	// Flush all changes to the configuration file.
	gnome_config_sync();

	// Stop the asynchronous DNS resolver.
	if(app_resolver_select_id != -1)
	{
		gdk_input_remove(app_resolver_select_id);
		app_resolver_select_id = -1;
	}
	if(app_resolver_idle_id != (guint)(-1))
	{
		gtk_idle_remove(app_resolver_idle_id);
		app_resolver_idle_id = (guint)(-1);
	}
	ResolverStop();

	// Quit the application.
	gtk_main_quit();
}

static void cb_capture_start(GtkWidget *widget)
{
	char *netname;
	char errbuf[PCAP_ERRBUF_SIZE];
	char *msg;

	// Clear the display windows.
	clearWindows();

	// Close the network monitor, if it is already open.
	if(app_input_select_fd != -1)
	{
		gdk_input_remove(app_input_select_fd);
		app_input_select_fd = -1;
	}
	if(global_net_monitor.isOpen())
	{
		global_net_monitor.closeDumper();
		global_net_monitor.closeCapture();
	}

	// Attempt to open the default network device.
	if(!global_net_monitor.openDevice(NULL, errbuf, pref_capture_length))
	{
		// Try opening the loopback device as the fallback.
		if(!global_net_monitor.openDevice("lo", errbuf, pref_capture_length))
		{
			if(!global_net_monitor.openDevice("lo0", errbuf,
											  pref_capture_length))
			{
				// Try the default device again to get a meaningful error msg.
				if(!global_net_monitor.openDevice(NULL, errbuf,
												  pref_capture_length))
				{
					netname = 0;
				}
				else
				{
					// By some miracle, the device opened the second time.
					netname = pcap_lookupdev(errbuf);
				}
			}
			else
			{
				netname = "lo0";
			}
		}
		else
		{
			netname = "lo";
		}
	}
	else
	{
		netname = pcap_lookupdev(errbuf);
	}
	setResolverInfo(netname);
	if(global_net_monitor.isOpen())
	{
		// Set the filter condition, if any.
		char *filter = global_filter.convert(global_net_monitor.getLinkType());
		if(filter)
		{
			if(!global_net_monitor.addFilter(filter, errbuf))
			{
				global_net_monitor.closeCapture();
				free(filter);
				msg = g_strdup_printf(_("Can not set packet filter: %s"),
							  		  errbuf);
				gnome_error_dialog_parented(msg, GTK_WINDOW(app_main_window));
				g_free(msg);
				menu_util_process_items(app_main_window, menu_states,
										toolbar_mappings, 0);
				return;
			}
			free(filter);
		}

		// Set up an input callback to handle the network packets.
		app_input_select_fd = gdk_input_add(global_net_monitor.getCaptureFD(),
					  					    GDK_INPUT_READ, cb_monitor_data,
											NULL);

		// Set the status line to indicate that a network device is open.
		msg = g_strdup_printf(_("Network device %s is open for packet capture"),
							  netname);
		gnome_appbar_set_default(GNOME_APPBAR(app_status_bar), msg);
		g_free(msg);
	}
	else
	{
		// Pop up a dialog to indicate an error.
		msg = g_strdup_printf(_("Can not capture network packets: %s"),
							  errbuf);
		gnome_error_dialog_parented(msg, GTK_WINDOW(app_main_window));
		g_free(msg);
	}

	// Update the menu and toolbar states.
	menu_util_process_items(app_main_window, menu_states, toolbar_mappings, 0);
}

static void cb_capture_stop(GtkWidget *widget)
{
	char *msg;

	// Close the network monitor, if it is already open.
	if(app_input_select_fd != -1)
	{
		gdk_input_remove(app_input_select_fd);
		app_input_select_fd = -1;
	}
	if(global_net_monitor.isOpen())
	{
		global_net_monitor.closeDumper();
		global_net_monitor.closeCapture();
	}

	// Report the final packet count.
	msg = g_strdup_printf(_("%lu packets received"), global_net_packet_count);
	gnome_appbar_set_default(GNOME_APPBAR(app_status_bar), msg);
	g_free(msg);

	// Update the menu and toolbar states.
	menu_util_process_items(app_main_window, menu_states, toolbar_mappings, 0);
}

static void cb_capture_clear(GtkWidget *widget)
{
	clearWindows();
	if(global_net_monitor.isOpen())
	{
		gnome_appbar_set_default(GNOME_APPBAR(app_status_bar),
								 _("0 packets received"));
	}
	else
	{
		gnome_appbar_set_default(GNOME_APPBAR(app_status_bar), "");
	}
	menu_util_process_items(app_main_window, menu_states, toolbar_mappings, 0);
}

static void cb_tools_ping(GtkWidget *widget)
{
	PingDialogBox *dlog = new PingDialogBox(GTK_WIDGET(app_main_window));
	dlog->show();
}

static void cb_tools_traceroute(GtkWidget *widget)
{
}

static void cb_tools_host_lookup(GtkWidget *widget)
{
	LookupDialogBox *dlog = new LookupDialogBox(GTK_WIDGET(app_main_window));
	dlog->show();
}

static void command_dialog(char *name, const char *command, const char *args)
{
	GtkWidget *dialog;
	GtkWidget *dialogbox;
	GtkWidget *less;

	// Construct the display dialog.
	dialog = gnome_dialog_new(name,
							  GNOME_STOCK_BUTTON_CLOSE,
							  NULL);
	gtk_window_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_policy(GTK_WINDOW(dialog), TRUE, TRUE, TRUE);
	gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);
	gnome_dialog_set_close(GNOME_DIALOG(dialog), TRUE);
	dialogbox = GNOME_DIALOG(dialog)->vbox;
	less = gnome_less_new();
	gnome_less_set_fixed_font(GNOME_LESS(less), TRUE);
	gtk_widget_set_usize(GTK_WIDGET(GNOME_LESS(less)->text), 600, 304);
	gtk_box_pack_start_defaults(GTK_BOX(dialogbox), less);
	gtk_widget_show_all(GTK_WIDGET(dialogbox));

	// Build the full command to be executed within the display window.
	char fullcmd[1024];
	sprintf(fullcmd, "%s %s", command, args);

	// Show the dialog, making it modeless in the process.
	gtk_widget_show(dialog);

	// Execute the command within the dialog.
	gnome_less_show_command(GNOME_LESS(less), fullcmd);
}

static GtkWidget *column_dialog(char *name, char **titles, int *widths,
								int numcols)
{
	GtkWidget *dialog;
	GtkWidget *dialogbox;
	GtkWidget *scroller;
	GtkWidget *clist;
	int col;

	// Construct the display dialog.
	dialog = gnome_dialog_new(name,
							  GNOME_STOCK_BUTTON_CLOSE,
							  NULL);
	gtk_window_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_policy(GTK_WINDOW(dialog), TRUE, TRUE, TRUE);
	gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);
	gnome_dialog_set_close(GNOME_DIALOG(dialog), TRUE);
	dialogbox = GNOME_DIALOG(dialog)->vbox;
	scroller = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroller),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC);
	clist = gtk_clist_new_with_titles(numcols, titles);
	gtk_container_add(GTK_CONTAINER(scroller), clist);
	gtk_clist_column_titles_passive(GTK_CLIST(clist));
	for(col = 0; col < numcols; ++col)
	{
		if(widths[col] < 0)
		{
			gtk_clist_set_column_justification(GTK_CLIST(clist), col,
											   GTK_JUSTIFY_RIGHT);
			gtk_clist_set_column_width(GTK_CLIST(clist), col,
									   -(widths[col] * 8));
		}
		else if(widths[col] > 0)
		{
			gtk_clist_set_column_width(GTK_CLIST(clist), col,
									   widths[col] * 8);
		}
	}
	gtk_widget_set_usize(GTK_WIDGET(scroller), 600, 304);
	gtk_box_pack_start_defaults(GTK_BOX(dialogbox), scroller);
	gtk_widget_show_all(GTK_WIDGET(dialogbox));

	// Show the dialog, making it modeless in the process.
	gtk_widget_show(dialog);

	// Return the column list widget to the caller.
	return clist;
}

static void cb_tools_network_interfaces(GtkWidget *widget)
{
	if(ifconfig_program != NULL)
	{
		command_dialog(_("Network interfaces"), ifconfig_program, "-a");
	}
}

#ifdef linux

static char *socket_states[] =
	{N_("Zero"), N_("Established"), N_("Sent"), N_("Recv"), N_("Wait 1"),
	 N_("Wait 2"), N_("Timed Wait"), N_("Close"), N_("Close Wait"),
	 N_("Last Ack"), N_("Listen"), N_("Closing")};

#endif

static void cb_tools_tcp_sockets(GtkWidget *widget)
{
#ifdef linux
	// On Linux, use the contents of "/proc/net/tcp" and a column
	// dialog to produce a more reasonable display than "netstat"
	// piped to a text widget can produce.
	static char *titles[] = {N_("Recv-Q"), N_("Send-Q"),
	                         N_("Local Address"), N_("Foreign Address"),
						     N_("State")};
	static int widths[] = {-6, -6, 22, 22, 0};
	GtkWidget *clist = column_dialog(_("Active TCP sockets"),
									 titles, widths, 5);
	ColumnFileParser parser;
	char **cols;
	int numcols;
	char *info[5];
	char tempbuf[1024];
	unsigned long port;
	gint row;
	struct servent *ent;
	gtk_clist_freeze(GTK_CLIST(clist));
	if(parser.openFile("/proc/net/tcp", 1))
	{
		parser.setSeparators(":");
		while((numcols = parser.readColumns(cols)) >= 0)
		{
			if(numcols < 14)
			{
				continue;
			}
			sprintf(tempbuf, "%lu", ColumnFileParser::hexToNum(cols[7]));
			sprintf(tempbuf + 32, "%lu", ColumnFileParser::hexToNum(cols[6]));
			port = ColumnFileParser::hexToNum(cols[2]);
			if(port)
			{
				ent = getservbyport((int)(htons(port)), "tcp");
				if(ent)
				{
					sprintf(tempbuf + 64, "%s:%s",
							ColumnFileParser::hexToIP(cols[1]),
							ent->s_name);
				}
				else
				{
					sprintf(tempbuf + 64, "%s:%lu",
							ColumnFileParser::hexToIP(cols[1]), port);
				}
			}
			else
			{
				sprintf(tempbuf + 64, "%s:*",
						ColumnFileParser::hexToIP(cols[1]));
			}
			port = ColumnFileParser::hexToNum(cols[4]);
			if(port)
			{
				ent = getservbyport((int)(htons(port)), "tcp");
				if(ent)
				{
					sprintf(tempbuf + 512, "%s:%s",
							ColumnFileParser::hexToIP(cols[3]),
							ent->s_name);
				}
				else
				{
					sprintf(tempbuf + 512, "%s:%lu",
							ColumnFileParser::hexToIP(cols[3]), port);
				}
			}
			else
			{
				sprintf(tempbuf + 512, "%s:*",
						ColumnFileParser::hexToIP(cols[3]));
			}
			info[0] = tempbuf;
			info[1] = tempbuf + 32;
			info[2] = tempbuf + 64;
			info[3] = tempbuf + 512;
			info[4] = socket_states[ColumnFileParser::hexToNum(cols[5])];
			row = gtk_clist_append(GTK_CLIST(clist), info);
			gtk_clist_set_selectable(GTK_CLIST(clist), row, FALSE);
		}
		parser.close();
	}
	gtk_clist_thaw(GTK_CLIST(clist));
#else
	// Other platforms: just display the output of "netstat".
	if(netstat_program != NULL)
	{
		command_dialog(_("Active TCP sockets"), netstat_program, "-t -a -l");
	}
#endif
}

static void cb_tools_udp_sockets(GtkWidget *widget)
{
#ifdef linux
	// On Linux, use the contents of "/proc/net/udp" and a column
	// dialog to produce a more reasonable display than "netstat"
	// piped to a text widget can produce.
	static char *titles[] = {N_("Recv-Q"), N_("Send-Q"),
	                         N_("Local Address"), N_("Foreign Address")};
	static int widths[] = {-6, -6, 29, 0};
	GtkWidget *clist = column_dialog(_("Active UDP sockets"),
									 titles, widths, 4);
	ColumnFileParser parser;
	char **cols;
	int numcols;
	char *info[4];
	char tempbuf[1024];
	unsigned long port;
	gint row;
	struct servent *ent;
	gtk_clist_freeze(GTK_CLIST(clist));
	if(parser.openFile("/proc/net/udp", 1))
	{
		parser.setSeparators(":");
		while((numcols = parser.readColumns(cols)) >= 0)
		{
			if(numcols < 14)
			{
				continue;
			}
			sprintf(tempbuf, "%lu", ColumnFileParser::hexToNum(cols[7]));
			sprintf(tempbuf + 32, "%lu", ColumnFileParser::hexToNum(cols[6]));
			port = ColumnFileParser::hexToNum(cols[2]);
			if(port)
			{
				ent = getservbyport((int)(htons(port)), "udp");
				if(ent)
				{
					sprintf(tempbuf + 64, "%s:%s",
							ColumnFileParser::hexToIP(cols[1]),
							ent->s_name);
				}
				else
				{
					sprintf(tempbuf + 64, "%s:%lu",
							ColumnFileParser::hexToIP(cols[1]), port);
				}
			}
			else
			{
				sprintf(tempbuf + 64, "%s:*",
						ColumnFileParser::hexToIP(cols[1]));
			}
			port = ColumnFileParser::hexToNum(cols[4]);
			if(port)
			{
				ent = getservbyport((int)(htons(port)), "udp");
				if(ent)
				{
					sprintf(tempbuf + 512, "%s:%s",
							ColumnFileParser::hexToIP(cols[3]),
							ent->s_name);
				}
				else
				{
					sprintf(tempbuf + 512, "%s:%lu",
							ColumnFileParser::hexToIP(cols[3]), port);
				}
			}
			else
			{
				sprintf(tempbuf + 512, "%s:*",
						ColumnFileParser::hexToIP(cols[3]));
			}
			info[0] = tempbuf;
			info[1] = tempbuf + 32;
			info[2] = tempbuf + 64;
			info[3] = tempbuf + 512;
			row = gtk_clist_append(GTK_CLIST(clist), info);
			gtk_clist_set_selectable(GTK_CLIST(clist), row, FALSE);
		}
		parser.close();
	}
	gtk_clist_thaw(GTK_CLIST(clist));
#else
	// Other platforms: just display the output of "netstat".
	if(netstat_program != NULL)
	{
		command_dialog(_("Active UDP sockets"), netstat_program, "-u -a -l");
	}
#endif
}

#ifdef linux

static char *socket_types[] =
	{N_("0"), N_("Stream"), N_("Datagram"), N_("Raw"),
	 N_("RDM"), N_("SeqPacket"), N_("6"), N_("7"),
	 N_("8"), N_("9"), N_("Packet")};

static char *socket_unix_states[] =
	{N_("Free"), N_("Listen"), N_("Connecting"), N_("Connected"),
	 N_("Disconnecting")};

#endif

static void cb_tools_unix_sockets(GtkWidget *widget)
{
#ifdef linux
	// On Linux, use the contents of "/proc/net/unix" and a column
	// dialog to produce a more reasonable display than "netstat"
	// piped to a text widget can produce.
	static char *titles[] = {N_("RefCnt"), N_("Flags"),
	                         N_("Type"), N_("State"), N_("I-Node"),
						     N_("Path")};
	static int widths[] = {-5, 6, 10, 11, -6, 0};
	GtkWidget *clist = column_dialog(_("Active Unix domain sockets"),
									 titles, widths, 6);
	ColumnFileParser parser;
	char **cols;
	int numcols;
	char *info[6];
	char tempbuf[1024];
	gint row;
	unsigned long flags;
	gtk_clist_freeze(GTK_CLIST(clist));
	if(parser.openFile("/proc/net/unix", 1))
	{
		while((numcols = parser.readColumns(cols)) >= 0)
		{
			if(numcols < 7)
			{
				continue;
			}
			sprintf(tempbuf, "%lu", ColumnFileParser::hexToNum(cols[1]));
			info[0] = tempbuf;
			tempbuf[64] = '\0';
			flags = ColumnFileParser::hexToNum(cols[3]);
			if((flags & 0x00010000) != 0)
			{
				strcat(tempbuf + 64, "Accept");
			}
			if((flags & 0x00020000) != 0)
			{
				if(tempbuf[64] == '\0')
					strcat(tempbuf + 64, "Data");
				else
					strcat(tempbuf + 64, ", Data");
			}
			if((flags & 0x00040000) != 0)
			{
				if(tempbuf[64] == '\0')
					strcat(tempbuf + 64, "No Space");
				else
					strcat(tempbuf + 64, ", No Space");
			}
			info[1] = tempbuf + 64;
			info[2] = socket_types[ColumnFileParser::hexToNum(cols[4])];
			info[3] = socket_unix_states[ColumnFileParser::hexToNum(cols[5])];
			info[4] = cols[6];
			if(numcols < 8)
				info[5] = "";
			else
				info[5] = cols[7];
			row = gtk_clist_append(GTK_CLIST(clist), info);
			gtk_clist_set_selectable(GTK_CLIST(clist), row, FALSE);
		}
		parser.close();
	}
	gtk_clist_thaw(GTK_CLIST(clist));
#else
	// Other platforms: just display the output of "netstat".
	if(netstat_program != NULL)
	{
		command_dialog(_("Active Unix domain sockets"),
					   netstat_program, "-x -a -l");
	}
#endif
}

static void cb_tools_routing_table(GtkWidget *widget)
{
	if(netstat_program != NULL)
	{
		command_dialog(_("Routing table"), netstat_program, "-r -n");
	}
}

static void cb_tools_arp_table(GtkWidget *widget)
{
	if(arp_program != NULL)
	{
		command_dialog(_("ARP table"), arp_program, "-n");
	}
}

static void cb_tools_rpc_services(GtkWidget *widget)
{
	if(rpcinfo_program != NULL)
	{
		static char *titles[] = {N_("Program"), N_("Version"),
		                         N_("Protocol"), N_("Port"), N_("Name")};
		static int widths[] = {-6, -5, -6, -5, 0};
		GtkWidget *clist = column_dialog(_("RPC services"),
										 titles, widths, 5);
		ColumnFileParser parser;
		char **cols;
		int numcols;
		gint row;
		char tempbuf[1024];
		strcpy(tempbuf, rpcinfo_program);
		strcat(tempbuf, " -p");
		gtk_clist_freeze(GTK_CLIST(clist));
		if(parser.openPipe(tempbuf, 1))
		{
			while((numcols = parser.readColumns(cols)) >= 0)
			{
				if(numcols < 5)
				{
					continue;
				}
				row = gtk_clist_append(GTK_CLIST(clist), cols);
				gtk_clist_set_selectable(GTK_CLIST(clist), row, FALSE);
			}
			parser.close();
		}
		gtk_clist_thaw(GTK_CLIST(clist));
	}
}

static void cb_settings_filter(GtkWidget *widget)
{
	GtkWidget *dialog;
	GtkWidget *dialogbox;
	GtkWidget *frame;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *addrNone;
	GtkWidget *addrTo;
	GtkWidget *addrToText;
	GtkWidget *addrFrom;
	GtkWidget *addrFromText;
	GtkWidget *addrToOrFrom;
	GtkWidget *addrToOrFromText;
	GtkWidget *addrBetween;
	GtkWidget *addrBetweenText;
	GtkWidget *addrAndText;
	GtkWidget *portNone;
	GtkWidget *portTo;
	GtkWidget *portToText;
	GtkWidget *portChoose;
	GtkWidget *protocolUDP;
	GtkWidget *protocolTCP;
	GtkWidget *protocolICMP;
	GtkWidget *protocolOtherIP;
	GtkWidget *protocolARP;
	GtkWidget *protocolRARP;
	GtkWidget *protocolOtherEthernet;
	GSList *group;
	FilterCondition *filter = &global_filter;

	// Create the filter dialog frame.
	dialog = gnome_dialog_new(_("Filtering conditions"),
						      GNOME_STOCK_BUTTON_OK,
							  GNOME_STOCK_BUTTON_CANCEL,
							  NULL);
	gtk_window_set_policy(GTK_WINDOW(dialog), TRUE, TRUE, TRUE);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog),
							GTK_WINDOW(app_main_window));
	gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);
	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);

	// Add the controls to the dialog.
	dialogbox = GNOME_DIALOG(dialog)->vbox;
	frame = gtk_frame_new(_("Address filter"));
	vbox = gtk_vbox_new(TRUE, 3);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	gtk_box_pack_start(GTK_BOX(dialogbox), frame, FALSE, FALSE, 0);

	addrNone = gtk_radio_button_new_with_label(NULL, _("No address filtering"));
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(addrNone));
	gtk_box_pack_start_defaults(GTK_BOX(vbox), addrNone);

	hbox = gtk_hbox_new(0, 0);
	addrTo = gtk_radio_button_new_with_label(group, _("To:"));
	gtk_box_pack_start(GTK_BOX(hbox), addrTo, FALSE, TRUE, 0);
	addrToText = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(addrToText), TRUE);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), addrToText);
	gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(addrTo));

	hbox = gtk_hbox_new(0, 0);
	addrFrom = gtk_radio_button_new_with_label(group, _("From:"));
	gtk_box_pack_start(GTK_BOX(hbox), addrFrom, FALSE, TRUE, 0);
	addrFromText = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(addrFromText), TRUE);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), addrFromText);
	gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(addrFrom));

	hbox = gtk_hbox_new(0, 0);
	addrToOrFrom = gtk_radio_button_new_with_label(group, _("To or from:"));
	gtk_box_pack_start(GTK_BOX(hbox), addrToOrFrom, FALSE, TRUE, 0);
	addrToOrFromText = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(addrToOrFromText), TRUE);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), addrToOrFromText);
	gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(addrToOrFrom));

	hbox = gtk_hbox_new(0, 0);
	addrBetween = gtk_radio_button_new_with_label(group, _("Between"));
	gtk_box_pack_start(GTK_BOX(hbox), addrBetween, FALSE, TRUE, 0);
	addrBetweenText = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(addrBetweenText), TRUE);
	gtk_box_pack_start(GTK_BOX(hbox), addrBetweenText, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("and"), FALSE, TRUE, 5);
	addrAndText = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(addrAndText), TRUE);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), addrAndText);
	gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);

	frame = gtk_frame_new(_("Protocol filter"));
	vbox = gtk_vbox_new(TRUE, 3);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	hbox = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
	gtk_box_pack_start(GTK_BOX(dialogbox), frame, FALSE, FALSE, 0);

	protocolUDP = gtk_check_button_new_with_label(_("UDP"));
	gtk_box_pack_start_defaults(GTK_BOX(hbox), protocolUDP);
	protocolTCP = gtk_check_button_new_with_label(_("TCP"));
	gtk_box_pack_start_defaults(GTK_BOX(hbox), protocolTCP);
	protocolICMP = gtk_check_button_new_with_label(_("ICMP"));
	gtk_box_pack_start_defaults(GTK_BOX(hbox), protocolICMP);
	protocolOtherIP = gtk_check_button_new_with_label(_("Other IP"));
	gtk_box_pack_start_defaults(GTK_BOX(hbox), protocolOtherIP);

	hbox = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
	protocolARP = gtk_check_button_new_with_label(_("ARP"));
	gtk_box_pack_start_defaults(GTK_BOX(hbox), protocolARP);
	protocolRARP = gtk_check_button_new_with_label(_("RARP"));
	gtk_box_pack_start_defaults(GTK_BOX(hbox), protocolRARP);
	protocolOtherEthernet =
		gtk_check_button_new_with_label(_("Other Ethernet"));
	gtk_box_pack_start_defaults(GTK_BOX(hbox), protocolOtherEthernet);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new(""));

	frame = gtk_frame_new(_("Port filter"));
	vbox = gtk_vbox_new(TRUE, 3);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	gtk_box_pack_start(GTK_BOX(dialogbox), frame, FALSE, FALSE, 0);

	portNone = gtk_radio_button_new_with_label(NULL, _("No port filtering"));
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(portNone));
	gtk_box_pack_start_defaults(GTK_BOX(vbox), portNone);

	hbox = gtk_hbox_new(0, 0);
	portTo = gtk_radio_button_new_with_label(group, _("Ports:"));
	gtk_box_pack_start(GTK_BOX(hbox), portTo, FALSE, TRUE, 0);
	portToText = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(portToText), TRUE);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), portToText);
#if 0
	portChoose = gtk_button_new_with_label(_("Choose..."));
	gtk_box_pack_start(GTK_BOX(hbox), portChoose, FALSE, TRUE, 5);
#endif
	gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);

	// Set the initial values of all of the fields.
	switch(filter->addrType)
	{
		case FilterAddressType_None:
		{
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(addrNone), TRUE);
		}
		break;

		case FilterAddressType_To:
		{
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(addrTo), TRUE);
		}
		break;

		case FilterAddressType_From:
		{
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(addrFrom), TRUE);
		}
		break;

		case FilterAddressType_ToOrFrom:
		{
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(addrToOrFrom), TRUE);
		}
		break;

		case FilterAddressType_Between:
		{
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(addrBetween), TRUE);
		}
		break;
	}
	if(filter->addrTo)
	{
		gtk_entry_set_text(GTK_ENTRY(addrToText), filter->addrTo);
	}
	if(filter->addrFrom)
	{
		gtk_entry_set_text(GTK_ENTRY(addrFromText), filter->addrFrom);
	}
	if(filter->addrToOrFrom)
	{
		gtk_entry_set_text(GTK_ENTRY(addrToOrFromText), filter->addrToOrFrom);
	}
	if(filter->addrBetween)
	{
		gtk_entry_set_text(GTK_ENTRY(addrBetweenText), filter->addrBetween);
	}
	if(filter->addrAnd)
	{
		gtk_entry_set_text(GTK_ENTRY(addrAndText), filter->addrAnd);
	}
	if(filter->hasProtocol(FilterProtocolType_UDP))
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(protocolUDP), TRUE);
	}
	if(filter->hasProtocol(FilterProtocolType_TCP))
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(protocolTCP), TRUE);
	}
	if(filter->hasProtocol(FilterProtocolType_ICMP))
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(protocolICMP), TRUE);
	}
	if(filter->hasProtocol(FilterProtocolType_OtherIP))
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(protocolOtherIP), TRUE);
	}
	if(filter->hasProtocol(FilterProtocolType_ARP))
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(protocolARP), TRUE);
	}
	if(filter->hasProtocol(FilterProtocolType_RARP))
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(protocolRARP), TRUE);
	}
	if(filter->hasProtocol(FilterProtocolType_OtherEthernet))
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(protocolOtherEthernet),
									 TRUE);
	}
	if(filter->portEnabled)
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(portTo), TRUE);
	}
	else
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(portNone), TRUE);
	}
	if(filter->ports)
	{
		gtk_entry_set_text(GTK_ENTRY(portToText), filter->ports);
	}

	// Set all controls within the dialog to "shown".
	gtk_widget_show_all(GTK_WIDGET(dialogbox));

	// Run the filter dialog box.  Note: if this function returns -1,
	// then the close button has been pressed on the dialogs's title bar,
	// or the user pressed Escape.  GNOME has already destroyed the
	// dialog widget in this case.
	int button = gnome_dialog_run(GNOME_DIALOG(dialog));
	if(button != -1)
	{
		if(!button)
		{
			// OK button was pressed on the dialog box.
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(addrNone)))
			{
				filter->addrType = FilterAddressType_None;
			}
			else if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(addrTo)))
			{
				filter->addrType = FilterAddressType_To;
			}
			else if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(addrFrom)))
			{
				filter->addrType = FilterAddressType_From;
			}
			else if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(addrToOrFrom)))
			{
				filter->addrType = FilterAddressType_ToOrFrom;
			}
			else if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(addrBetween)))
			{
				filter->addrType = FilterAddressType_Between;
			}
			filter->setAddrTo(gtk_entry_get_text(GTK_ENTRY(addrToText)));
			filter->setAddrFrom(gtk_entry_get_text(GTK_ENTRY(addrFromText)));
			filter->setAddrToOrFrom(gtk_entry_get_text(GTK_ENTRY(addrToOrFromText)));
			filter->setAddrBetween(gtk_entry_get_text(GTK_ENTRY(addrBetweenText)));
			filter->setAddrAnd(gtk_entry_get_text(GTK_ENTRY(addrAndText)));
			filter->protocols = FilterProtocolType_None;
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(protocolUDP)))
			{
				filter->setProtocol(FilterProtocolType_UDP);
			}
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(protocolTCP)))
			{
				filter->setProtocol(FilterProtocolType_TCP);
			}
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(protocolICMP)))
			{
				filter->setProtocol(FilterProtocolType_ICMP);
			}
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(protocolOtherIP)))
			{
				filter->setProtocol(FilterProtocolType_OtherIP);
			}
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(protocolARP)))
			{
				filter->setProtocol(FilterProtocolType_ARP);
			}
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(protocolRARP)))
			{
				filter->setProtocol(FilterProtocolType_RARP);
			}
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(protocolOtherEthernet)))
			{
				filter->setProtocol(FilterProtocolType_OtherEthernet);
			}
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(portNone)))
			{
				filter->portEnabled = 0;
			}
			else
			{
				filter->portEnabled = 1;
			}
			filter->setPorts(gtk_entry_get_text(GTK_ENTRY(portToText)));
		}
		gtk_widget_destroy(dialog);
	}
}

static void cb_settings_prefs(GtkWidget *widget)
{
	GtkWidget *dialog;
	GtkWidget *dialogbox;
	GtkWidget *frame;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *resolve;
	GtkWidget *foreign;
	GtkWidget *fqdn;
	GtkWidget *autoscroll;
	GtkWidget *detailed;
	GtkWidget *hex;
	GtkWidget *captureLen;
	GSList *group;
	char numbuf[32];

	// Create the preferences dialog frame.
	dialog = gnome_dialog_new(_("Preferences"),
						      GNOME_STOCK_BUTTON_OK,
							  GNOME_STOCK_BUTTON_CANCEL,
							  NULL);
	gtk_window_set_policy(GTK_WINDOW(dialog), TRUE, TRUE, TRUE);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog),
							GTK_WINDOW(app_main_window));
	gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);
	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
	dialogbox = GNOME_DIALOG(dialog)->vbox;

	// Add the controls to the dialog box.
	resolve = gtk_check_button_new_with_label
					(_("Resolve IP addresses into domain names"));
	gtk_box_pack_start_defaults(GTK_BOX(dialogbox), resolve);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(resolve),
								 pref_resolve_addresses);

	foreign = gtk_check_button_new_with_label
					(_("Resolve IP addresses on foreign subnets"));
	gtk_box_pack_start_defaults(GTK_BOX(dialogbox), foreign);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(foreign),
								 pref_resolve_foreign);

	fqdn = gtk_check_button_new_with_label
					(_("Show fully qualified domain names"));
	gtk_box_pack_start_defaults(GTK_BOX(dialogbox), fqdn);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fqdn),
								 pref_fully_qualified);

	autoscroll = gtk_check_button_new_with_label
					(_("Auto-scroll when new packets arrive"));
	gtk_box_pack_start_defaults(GTK_BOX(dialogbox), autoscroll);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autoscroll),
								 pref_auto_scroll);

	frame = gtk_frame_new(_("Packet display"));
	vbox = gtk_vbox_new(TRUE, 3);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	gtk_box_pack_start(GTK_BOX(dialogbox), frame, FALSE, FALSE, 0);

	detailed = gtk_radio_button_new_with_label(NULL,
						_("Detailed field descriptions"));
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(detailed));
	gtk_box_pack_start_defaults(GTK_BOX(vbox), detailed);
	hex = gtk_radio_button_new_with_label(group,
						_("Raw hexadecimal packet dump"));
	gtk_box_pack_start_defaults(GTK_BOX(vbox), hex);
	if(pref_hex_packets)
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(hex), TRUE);
	}
	else
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(detailed), TRUE);
	}

	hbox = gtk_hbox_new(0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Capture length:")),
					   FALSE, TRUE, 5);
	captureLen = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(captureLen), TRUE);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), captureLen);
	gtk_box_pack_start_defaults(GTK_BOX(dialogbox), hbox);
	sprintf(numbuf, "%d", pref_capture_length);
	gtk_entry_set_text(GTK_ENTRY(captureLen), numbuf);

	// Set all controls within the dialog to "shown".
	gtk_widget_show_all(GTK_WIDGET(dialogbox));

	// Run the preferences dialog box.  Note: if this function returns -1,
	// then the close button has been pressed on the dialogs's title bar,
	// or the user pressed Escape.  GNOME has already destroyed the
	// dialog widget in this case.
	int button = gnome_dialog_run(GNOME_DIALOG(dialog));
	if(button != -1)
	{
		if(!button)
		{
			// OK button was pressed on the dialog box.
			int oldhex = pref_hex_packets;
			pref_resolve_addresses =
				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resolve));
			pref_resolve_foreign =
				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(foreign));
			pref_fully_qualified =
				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fqdn));
			pref_auto_scroll =
				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(autoscroll));
			pref_hex_packets =
				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(hex));
			pref_capture_length =
				atoi(gtk_entry_get_text(GTK_ENTRY(captureLen)));
			if(pref_capture_length < 68)
			{
				pref_capture_length = 68;
			}
			if(oldhex != pref_hex_packets &&
			   app_short_form_selected != -1)
			{
				// Re-display the long form packet in the new format.
				cb_select_packet(GTK_CLIST(app_short_form),
								 app_short_form_selected, 0, 0, 0);
			}
			ResolverChangeForeign(pref_resolve_foreign);
		}
		gtk_widget_destroy(dialog);
	}
}

static void cb_help_about(GtkWidget *widget)
{
	static const gchar *authors[] = {"Rhys Weatherley", NULL};
	GtkWidget *about =
		gnome_about_new(NETPEEK_NAME_DISPLAY, NETPEEK_NAME_VERSION,
						_("Copyright (C) 1999 Rhys Weatherley"),
						authors,
						_("Network monitoring and diagnosis tool."),
						NULL);
	gtk_widget_show(about);		
}

static gboolean cb_file_open_enabled(void *data)
{
	return TRUE;
}

static gboolean cb_file_save_enabled(void *data)
{
	return (!global_net_monitor.isOpen() && global_net_packet_count > 0);
}

static gboolean cb_file_print_enabled(void *data)
{
	return (!global_net_monitor.isOpen() && global_net_packet_count > 0);
}

static gboolean cb_capture_start_enabled(void *data)
{
	return !global_net_monitor.isOpen();
}

static gboolean cb_capture_stop_enabled(void *data)
{
	return global_net_monitor.isOpen();
}

static gboolean cb_capture_clear_enabled(void *data)
{
	return (global_net_packet_count > 0);
}

static gboolean cb_tools_ping_enabled(GtkWidget *widget)
{
	static gboolean tested_ping = FALSE;
	static gboolean test_result;
	if(!tested_ping)
	{
		test_result = (gboolean)(PingSupport::canPing());
		tested_ping = TRUE;
	}
	return test_result;
}

static gboolean cb_tools_traceroute_enabled(GtkWidget *widget)
{
	return FALSE;
}

static gboolean cb_tools_host_lookup_enabled(GtkWidget *widget)
{
	return TRUE;
}

static gboolean cb_tools_network_interfaces_enabled(GtkWidget *widget)
{
	return (ifconfig_program != NULL);
}

static gboolean cb_tools_tcp_sockets_enabled(GtkWidget *widget)
{
	return (netstat_program != NULL);
}

static gboolean cb_tools_udp_sockets_enabled(GtkWidget *widget)
{
	return (netstat_program != NULL);
}

static gboolean cb_tools_unix_sockets_enabled(GtkWidget *widget)
{
	return (netstat_program != NULL);
}

static gboolean cb_tools_routing_table_enabled(GtkWidget *widget)
{
	return (netstat_program != NULL);
}

static gboolean cb_tools_arp_table_enabled(GtkWidget *widget)
{
	return (arp_program != NULL);
}

static gboolean cb_tools_rpc_services_enabled(GtkWidget *widget)
{
	return (rpcinfo_program != NULL);
}

static gboolean cb_settings_filter_enabled(void *data)
{
	return !global_net_monitor.isOpen();
}

/* GMT to local conversion routine extracted from tcpdump */

/*
 * Copyright (c) 1997
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the University of California,
 * Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/*
 * Returns the difference between gmt and local time in seconds.
 * Use gmtime() and localtime() to keep things simple.
 */
int32_t
gmt2local(time_t t)
{
	register int dt, dir;
	register struct tm *gmt, *loc;
	struct tm sgmt;

	if (t == 0)
		t = time(NULL);
	gmt = &sgmt;
	*gmt = *gmtime(&t);
	loc = localtime(&t);
	dt = (loc->tm_hour - gmt->tm_hour) * 60 * 60 +
	    (loc->tm_min - gmt->tm_min) * 60;

	/*
	 * If the year or julian day is different, we span 00:00 GMT
	 * and must add or subtract a day. Check the year first to
	 * avoid problems when the julian day wraps.
	 */
	dir = loc->tm_year - gmt->tm_year;
	if (dir == 0)
		dir = loc->tm_yday - gmt->tm_yday;
	dt += dir * 24 * 60 * 60;

	return (dt);
}
